@@ -41,7 +41,7 @@
  * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
  *
  *
- * $Id: BuilderConfiguration.java 8325 2009-07-30 16:31:32Z rjoachim $
+ * $Id: BuilderConfiguration.java 8600 2010-05-12 20:01:02Z wguttmn $
  */
 package org.exolab.castor.builder;
 
@@ -72,7 +72,7 @@ import org.castor.xml.XMLProperties;
  *
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
  * @author <a href="mailto:blandin@intalio.com">Arnaud Blandin</a>
- * @version $Revision: 8325 $ $Date: 2006-04-25 15:08:23 -0600 (Tue, 25 Apr 2006) $
+ * @version $Revision: 8600 $ $Date: 2006-04-25 15:08:23 -0600 (Tue, 25 Apr 2006) $
  */
 public class BuilderConfiguration {
     /**
@@ -284,6 +284,16 @@ public class BuilderConfiguration {
          */
         public static final String EXTRA_DOCUMENTATION_METHODS = 
             "org.exolab.castor.builder.extraDocumentationMethods";
+
+        /**
+         * Property specifying whether cycle breaker code should be added to
+         * generated methods 'equals' and 'hashcode'; defaults to <i>true</i>.
+         * 
+         * <pre>
+         * org.exolab.castor.builder.useCycleBreaker = true
+         * </pre>
+         */
+        public static final String USE_CYCLE_BREAKER = "org.exolab.castor.builder.useCycleBreaker";
         
     } //--Property
 
@@ -521,7 +531,17 @@ public class BuilderConfiguration {
             _localProps.getProperty(Property.EXTRA_DOCUMENTATION_METHODS, "false");
         return new Boolean(extraDocumentationMethods).booleanValue();
     }
-    
+
+    /**
+     * Returns true if the class {@link CycleBreaker} should be used during code generation; 
+     * defaults to 'true'.
+     *
+     * @return true if the class {@link CycleBreaker} should be used during code generation
+     */
+    public final boolean useCycleBreaker() {
+        return Boolean.valueOf(_localProps.getProperty(Property.USE_CYCLE_BREAKER, "true"));
+    }
+
     /**
      * Returns the maximum number of static constant definitions that are
      * acceptable within one class file; default is 1000.
