@@ -45,7 +45,7 @@
  * All portions of this file developed by Keith Visco after Jan 19 2005 are
  * Copyright (C) 2005 Keith Visco. All Rights Reserved.
  *
- * $Id: SourceGenerator.java 8405 2009-08-27 21:18:28Z wguttmn $
+ * $Id: SourceGenerator.java 8667 2010-08-14 22:24:49Z wguttmn $
  */
 package org.exolab.castor.builder;
 
@@ -56,6 +56,7 @@ import java.io.FileWriter;
 import java.io.IOException;
 import java.io.Reader;
 import java.util.Enumeration;
+import java.util.Iterator;
 import java.util.Properties;
 import java.util.Vector;
 
@@ -118,7 +119,7 @@ import org.xml.sax.SAXParseException;
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a> - Main author.
  * @author <a href="mailto:blandin@intalio.com">Arnaud Blandin</a> - Contributions.
  * @author <a href="mailto:nsgreen@thazar.com">Nathan Green</a> - Contributions.
- * @version $Revision: 8405 $ $Date: 2006-03-30 14:58:45 -0700 (Thu, 30 Mar 2006) $
+ * @version $Revision: 8667 $ $Date: 2006-03-30 14:58:45 -0700 (Thu, 30 Mar 2006) $
  */
 public class SourceGenerator extends BuilderConfiguration {
 
@@ -792,33 +793,31 @@ public class SourceGenerator extends BuilderConfiguration {
         //-- ** Generate code for all TOP-LEVEL structures **
 
         //-- register all global element names for name conflict resolution
-        for (Enumeration<ElementDecl> structures = schema.getElementDecls(); structures.hasMoreElements(); ) {
-            ElementDecl element = structures.nextElement();
+        for (ElementDecl element : schema.getElementDecls()) {
             _xmlInfoRegistry.prebindGlobalElement(XPathHelper.getSchemaLocation(element));
         }
-        for (Enumeration<ModelGroup> structures = schema.getModelGroups(); structures.hasMoreElements(); ) {
-            ModelGroup modelGroup = structures.nextElement();
+        for (ModelGroup modelGroup : schema.getModelGroups()) {
             _xmlInfoRegistry.prebindGlobalElement(XPathHelper.getSchemaLocation(modelGroup));
         }
 
         //-- handle all top-level element declarations
-        for (Enumeration<ElementDecl> structures = schema.getElementDecls(); structures.hasMoreElements(); ) {
-            createClasses(structures.nextElement(), sInfo);
+        for (ElementDecl element : schema.getElementDecls()) {
+            createClasses(element, sInfo);
         }
 
         //-- handle all top-level complextypes
-        for (Enumeration<ComplexType> structures = schema.getComplexTypes(); structures.hasMoreElements(); ) {
-            processComplexType(structures.nextElement(), sInfo);
+        for (ComplexType complexType : schema.getComplexTypes()) {
+            processComplexType(complexType, sInfo);
         }
 
         //-- handle all top-level simpletypes
-        for (Enumeration<SimpleType> structures = schema.getSimpleTypes(); structures.hasMoreElements(); ) {
-            processSimpleType(structures.nextElement(), sInfo);
+        for (SimpleType simpleType : schema.getSimpleTypes()) {
+            processSimpleType(simpleType, sInfo);
         }
 
         //-- handle all top-level groups
-        for (Enumeration<ModelGroup> structures = schema.getModelGroups(); structures.hasMoreElements(); ) {
-            createClasses(structures.nextElement(), sInfo);
+        for (ModelGroup modelGroup : schema.getModelGroups()) {
+            createClasses(modelGroup, sInfo);
         }
 
         //-- clean up any remaining JClasses which need printing
@@ -844,9 +843,9 @@ public class SourceGenerator extends BuilderConfiguration {
      */
     private void processImportedSchemas(final Schema schema, final SGStateInfo sInfo)
     throws IOException {
-        Enumeration<Schema> enumeration = schema.getImportedSchema();
-        while (enumeration.hasMoreElements()) {
-            Schema importedSchema = enumeration.nextElement();
+        Iterator<Schema> enumeration = schema.getImportedSchema().iterator();
+        while (enumeration.hasNext()) {
+            Schema importedSchema = enumeration.next();
             if (!_generateImported) {
                 LOG.warn("Note: No code will be generated for the following *imported* schema: " 
                         + importedSchema.getSchemaLocation() + "; if this is on intention, please "
