@@ -24,7 +24,7 @@ package org.castor.cpa.persistence.sql.query;
  */
 public final class Table extends Qualifier {
     //-----------------------------------------------------------------------------------    
-    
+
     /**
      * Construct a table with given name.
      * 
@@ -35,4 +35,11 @@ public final class Table extends Qualifier {
     }
 
     //-----------------------------------------------------------------------------------    
+
+    /**
+     * {@inheritDoc}
+     */
+    public void accept (final Visitor visitor) { visitor.visit(this); }
+
+    //-----------------------------------------------------------------------------------    
 }
