@@ -21,6 +21,7 @@ import java.util.Properties;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.castor.jdo.conf.ClassMapping;
 import org.castor.jdo.conf.DataSource;
 import org.castor.jdo.conf.Database;
 import org.castor.jdo.conf.DatabaseChoice;
@@ -28,6 +29,7 @@ import org.castor.jdo.conf.Driver;
 import org.castor.jdo.conf.JdoConf;
 import org.castor.jdo.conf.Jndi;
 import org.castor.jdo.conf.Mapping;
+import org.castor.jdo.conf.PackageMapping;
 import org.castor.jdo.conf.Param;
 import org.castor.jdo.conf.TransactionDemarcation;
 import org.castor.jdo.conf.TransactionManager;
@@ -63,7 +65,7 @@ import org.xml.sax.InputSource;
  *
  * @author <a href="mailto:martin-fuchs AT gmx DOT net">Martin Fuchs</a>
  * @author <a href="mailto:ralf DOT joachim AT syscon DOT eu">Ralf Joachim</a>
- * @version $Revision: 8104 $ $Date: 2005-11-19 10:17:16 -0700 (Sat, 19 Nov 2005) $
+ * @version $Revision: 8702 $ $Date: 2005-11-19 10:17:16 -0700 (Sat, 19 Nov 2005) $
  * @since 0.9.9.1
  */
 public final class JDOConfFactory {
@@ -81,6 +83,7 @@ public final class JDOConfFactory {
     /**
      * Creates a JdoConf instance from a SAX InputSource, using a Castor XML
      * Unmarshaller.
+     * 
      * @param source SAX input source representing the JDO configuration.
      * @param resolver SAX entity resolver.
      * @param loader Class loader.
@@ -155,44 +158,6 @@ public final class JDOConfFactory {
         return jdoConf;
     }
 
-    /**
-     * Create a database configuration with given name, engine and datasource
-     * configuration.
-     * 
-     * @param name Name of the database configuration.
-     * @param engine Name of the database engine.
-     * @param ds Datasource configuration.
-     * @param mapping Mapping configurations.
-     * @return Database configuration.
-     * @deprecated Pass mapping URL's to createDatabase() methods instead.
-     */
-    public static Database createDatabase(final String name, final String engine,
-            final DataSource ds, final Mapping mapping) {
-        return createDatabase(name, engine, ds, new Mapping[] {mapping});
-    }
-
-    /**
-     * Create a database configuration with given name, engine and datasource
-     * configuration.
-     * 
-     * @param name Name of the database configuration.
-     * @param engine Name of the database engine.
-     * @param ds Datasource configuration.
-     * @param mappings Array of mapping configurations.
-     * @return Database configuration.
-     * @deprecated Pass mapping URL's to createDatabase() methods instead.
-     */
-    public static Database createDatabase(final String name, final String engine,
-            final DataSource ds, final Mapping[] mappings) {
-        DatabaseChoice dbChoice = new DatabaseChoice();
-        dbChoice.setDataSource(ds);
-
-        Database dbConf = createDatabase(name, engine);
-        dbConf.setDatabaseChoice(dbChoice);
-        dbConf.setMapping(mappings);
-        return dbConf;
-    }
-
     /**
      * Create a database configuration with given name, engine and datasource
      * configuration.
@@ -202,6 +167,7 @@ public final class JDOConfFactory {
      * @param ds Datasource configuration.
      * @param mapping Mapping URL.
      * @return Database configuration.
+     * @deprecated create database w/o mapping.  
      */
     public static Database createDatabase(final String name, final String engine,
             final DataSource ds, final String mapping) {
@@ -217,6 +183,7 @@ public final class JDOConfFactory {
      * @param ds Datasource configuration.
      * @param mappings Array of mapping URL's.
      * @return Database configuration.
+     * @deprecated create database w/o mappings.  
      */
     public static Database createDatabase(final String name, final String engine,
             final DataSource ds, final String[] mappings) {
@@ -230,40 +197,22 @@ public final class JDOConfFactory {
     }
 
     /**
-     * Create a database configuration with given name, engine and driver
+     * Create a database configuration with given name, engine and datasource
      * configuration.
      * 
      * @param name Name of the database configuration.
      * @param engine Name of the database engine.
-     * @param driver Driver configuration.
-     * @param mapping Mapping configurations.
+     * @param ds Datasource configuration.
      * @return Database configuration.
-     * @deprecated Pass mapping URL's to createDatabase() methods instead.
+     * @since 1.3.2
      */
     public static Database createDatabase(final String name, final String engine,
-            final Driver driver, final Mapping mapping) {
-        return createDatabase(name, engine, driver, new Mapping[] {mapping});
-    }
-
-    /**
-     * Create a database configuration with given name, engine and driver
-     * configuration.
-     * 
-     * @param name Name of the database configuration.
-     * @param engine Name of the database engine.
-     * @param driver Driver configuration.
-     * @param mappings Array of mapping configurations.
-     * @return Database configuration.
-     * @deprecated Pass mapping URL's to createDatabase() methods instead.
-     */
-    public static Database createDatabase(final String name, final String engine,
-            final Driver driver, final Mapping[] mappings) {
-        DatabaseChoice dbChoise = new DatabaseChoice();
-        dbChoise.setDriver(driver);
+            final DataSource ds) {
+        DatabaseChoice dbChoice = new DatabaseChoice();
+        dbChoice.setDataSource(ds);
 
         Database dbConf = createDatabase(name, engine);
-        dbConf.setDatabaseChoice(dbChoise);
-        dbConf.setMapping(mappings);
+        dbConf.setDatabaseChoice(dbChoice);
         return dbConf;
     }
 
@@ -276,6 +225,7 @@ public final class JDOConfFactory {
      * @param driver Driver configuration.
      * @param mapping Mapping URL.
      * @return Database configuration.
+     * @deprecated create database w/o mapping.  
      */
     public static Database createDatabase(final String name, final String engine,
             final Driver driver, final String mapping) {
@@ -291,6 +241,7 @@ public final class JDOConfFactory {
      * @param driver Driver configuration.
      * @param mappings Array of mapping URL's.
      * @return Database configuration.
+     * @deprecated create database w/o mappings.  
      */
     public static Database createDatabase(final String name, final String engine,
             final Driver driver, final String[] mappings) {
@@ -303,6 +254,26 @@ public final class JDOConfFactory {
         return dbConf;
     }
 
+    /**
+     * Create a database configuration with given name, engine and driver
+     * configuration.
+     * 
+     * @param name Name of the database configuration.
+     * @param engine Name of the database engine.
+     * @param driver Driver configuration.
+     * @return Database configuration.
+     * @since 1.3.2
+     */
+    public static Database createDatabase(final String name, final String engine,
+            final Driver driver) {
+        DatabaseChoice dbChoise = new DatabaseChoice();
+        dbChoise.setDriver(driver);
+
+        Database dbConf = createDatabase(name, engine);
+        dbConf.setDatabaseChoice(dbChoise);
+        return dbConf;
+    }
+
     /**
      * Create a database configuration with given name, engine and JNDI
      * configuration.
@@ -312,6 +283,7 @@ public final class JDOConfFactory {
      * @param jndi JNDI configuration.
      * @param mapping Mapping URL.
      * @return Database configuration.
+     * @deprecated create database w/o mapping.  
      */
     public static Database createDatabase(final String name, final String engine,
             final Jndi jndi, final String mapping) {
@@ -327,6 +299,7 @@ public final class JDOConfFactory {
      * @param jndi JNDI configuration.
      * @param mappings Array of mapping URL's.
      * @return Database configuration.
+     * @deprecated create database w/o mappings.  
      */
     public static Database createDatabase(final String name, final String engine,
             final Jndi jndi, final String[] mappings) {
@@ -339,6 +312,26 @@ public final class JDOConfFactory {
         return dbConf;
     }
 
+    /**
+     * Create a database configuration with given name, engine and JNDI
+     * configuration.
+     * 
+     * @param name Name of the database configuration.
+     * @param engine Name of the database engine.
+     * @param jndi JNDI configuration.
+     * @return Database configuration.
+     * @since 1.3.2
+     */
+    public static Database createDatabase(final String name, final String engine,
+            final Jndi jndi) {
+        DatabaseChoice dbChoise = new DatabaseChoice();
+        dbChoise.setJndi(jndi);
+
+        Database dbConf = createDatabase(name, engine);
+        dbConf.setDatabaseChoice(dbChoise);
+        return dbConf;
+    }
+
     /**
      * Helper to create a database configuration with given name and engine.
      * 
@@ -408,24 +401,12 @@ public final class JDOConfFactory {
         return jndi;
     }
 
-    /**
-     * Create a JDO mapping configuration from given URL.
-     * 
-     * @param mapping URL to retrieve mapping configuration file.
-     * @return JDO Mapping configuration.
-     * @deprecated Pass mapping URL's to createDatabase() methods instead.
-     */
-    public static Mapping createMapping(final String mapping) {
-        Mapping mapConf = new Mapping();
-        mapConf.setHref(mapping);
-        return mapConf;
-    }
-
     /**
      * Create an array of JDO mapping configurations from given array of URL's.
      * 
      * @param mappings Array of URL's to retrieve mapping configuration file.
      * @return Array of JDO Mapping configurations.
+     * @deprecated
      */
     private static Mapping[] createMappings(final String[] mappings) {
         Mapping[] mapConfs = new Mapping[mappings.length];
@@ -437,13 +418,39 @@ public final class JDOConfFactory {
     }
 
     /**
-     * Create a transaction demarcation configuration with local transaction handling.
+     * Create an XML mapping configurations for JDO from given URL.
      * 
-     * @return TransactionDemarcation configuration with local transaction handling.
-     * @deprecated Use createLocalTransactionDemarcation() instead.
+     * @param mapping URL to retrieve mapping configuration file.
+     * @return XML mapping configuration for JDO.
      */
-    public static TransactionDemarcation createTransactionDemarcation() {
-        return createLocalTransactionDemarcation();
+    public static Mapping createXmlMapping(final String mapping) {
+        Mapping mapConf = new Mapping();
+        mapConf.setHref(mapping);
+        return mapConf;
+    }
+
+    /**
+     * Create an class mapping configurations for JDO from given class name.
+     * 
+     * @param classname Class name to retrieve mapping information.
+     * @return Class mapping configuration for JDO.
+     */
+    public static ClassMapping createClassMapping(final String classname) {
+        ClassMapping mapConf = new ClassMapping();
+        mapConf.setName(classname);
+        return mapConf;
+    }
+
+    /**
+     * Create an package mapping configurations for JDO from given package name.
+     * 
+     * @param packagename Package name to retrieve mapping information.
+     * @return Package mapping configuration for JDO.
+     */
+    public static PackageMapping createPackageMapping(final String packagename) {
+    	PackageMapping mapConf = new PackageMapping();
+        mapConf.setName(packagename);
+        return mapConf;
     }
 
     /**
