@@ -63,7 +63,7 @@ import org.exolab.castor.persist.spi.PersistenceQuery;
  * @author <a href="mailto:ralf DOT joachim AT syscon DOT eu">Ralf Joachim</a>
  * @author <a href="mailto:werner DOT guttmann AT gmx DOT net">Werner Guttmann</a>
  * @author <a href="mailto:gblock AT ctoforaday DOT COM">Gregory Block</a>
- * @version $Revision: 8471 $ $Date: 2006-04-25 15:08:23 -0600 (Tue, 25 Apr 2006) $
+ * @version $Revision: 8660 $ $Date: 2006-04-25 15:08:23 -0600 (Tue, 25 Apr 2006) $
  * @since 1.0
  */
 public abstract class AbstractTransactionContext implements TransactionContext {
@@ -229,6 +229,10 @@ public abstract class AbstractTransactionContext implements TransactionContext {
     public final void setInstanceFactory(final InstanceFactory factory) {
         _instanceFactory = factory;
     }
+    
+    public final InstanceFactory getInstanceFactory() {
+        return _instanceFactory;
+    }
 
     /**
      * {@inheritDoc}
@@ -538,10 +542,6 @@ public abstract class AbstractTransactionContext implements TransactionContext {
                         "persist.lockConflict", molder.getName(), identity));
             }
             
-            proposedObject.setProposedEntityClass(objectInTx.getClass());
-            proposedObject.setActualEntityClass(objectInTx.getClass());
-            proposedObject.setEntity(objectInTx);
-            
             return objectInTx;
         }
 
@@ -550,25 +550,8 @@ public abstract class AbstractTransactionContext implements TransactionContext {
         // failure (object no longer exists), hold until a suitable lock is granted
         // (or fail to grant), or report error with the persistence engine.
         try {
-            // Check whether an instance was given to the load method.
-            if (proposedObject.getEntity() != null) {
-                objectInTx = proposedObject.getEntity();
-            } else if (_instanceFactory != null) {
-                objectInTx = _instanceFactory.newInstance(molder.getName(), _db.getClassLoader());
-            } else {
-                objectInTx = molder.newInstance(_db.getClassLoader());
-            }
-            
-            molder.setIdentity(this, objectInTx, identity);
-            
-            proposedObject.setProposedEntityClass(objectInTx.getClass());
-            proposedObject.setActualEntityClass(objectInTx.getClass());
-            proposedObject.setEntity(objectInTx);
-
-            trackObject(molder, oid, proposedObject.getEntity());
-            
             engine.load(this, oid, proposedObject, suggestedAccessMode,
-                    _lockTimeout, results, molder);
+                    _lockTimeout, results, molder, identity);
         } catch (ClassCastException except) {
             _tracker.untrackObject(proposedObject.getEntity());
             throw except;
@@ -673,6 +656,7 @@ public abstract class AbstractTransactionContext implements TransactionContext {
         // if autoStore is specified, we relieve user life a little bit here
         // so that if an object create automatically and user create it
         // again, it won't receive exception
+        // TODO[ASE]: this autostore should be changed to cascading someday!
         if (_autoStore && _tracker.isTracking(object)) {
             return;
         }
@@ -788,6 +772,7 @@ public abstract class AbstractTransactionContext implements TransactionContext {
                         toBeCreatedMolder.getCallback().creating(toBeCreated, _db);
                     }
 
+                    // TODO[ASE]:here the problems begin with M:N(see inside to learn more)
                     OID oid = toBeCreatedLockEngine.create(this,
                             toBeCreatedOID, toBeCreated);
 
@@ -985,6 +970,7 @@ public abstract class AbstractTransactionContext implements TransactionContext {
         // Must acquire a write lock on the object in order to delete it,
         // prevents object form being deleted while someone else is
         // looking at it.
+        // TODO[ASE]: somewhere here we should define the cascading delete operation but how and where?
         try {
             _tracker.markDeleted(object);
             engine.softLock(this, oid, _lockTimeout);
@@ -1220,6 +1206,25 @@ public abstract class AbstractTransactionContext implements TransactionContext {
             LockEngine engine = molder.getLockEngine();
             OID oid = _tracker.getOIDForObject(toPrepare);
 
+            // Do the `modifying` callbacks
+            if (!isDeleted && !isCreating && needsPersist && needsCache) {
+                if (_callback != null) {
+                    try {
+                        _callback.modifying(toPrepare);
+                    } catch (Exception e) {
+                        throw new TransactionAbortedException(Messages.format(
+                                "persist.nested", e), e);
+                    }
+                } else if (molder.getCallback() != null) {
+                    try {
+                        molder.getCallback().modifying(toPrepare);
+                    } catch (Exception e) {
+                        throw new TransactionAbortedException(Messages.format(
+                                "persist.nested", e), e);
+                    }
+                }
+            }
+
             if (!isDeleted && !isCreating) {
                 if (needsPersist) {
                     engine.store(this, oid, toPrepare);
@@ -1229,7 +1234,7 @@ public abstract class AbstractTransactionContext implements TransactionContext {
                 }
             }
 
-            // do the callback
+            // Do the `storing` callbacks
             if (!isDeleted && _callback != null) {
                 try {
                     _callback.storing(toPrepare, needsCache);
