@@ -40,7 +40,7 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: Database.java 8351 2009-08-06 16:10:44Z lukaslang $
+ * $Id: Database.java 8660 2010-08-07 21:06:28Z wguttmn $
  */
 
 
@@ -97,7 +97,7 @@ import org.exolab.castor.persist.spi.Identity;
  * </pre>
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 8351 $ $Date: 2006-03-16 16:04:24 -0700 (Thu, 16 Mar 2006) $
+ * @version $Revision: 8660 $ $Date: 2006-03-16 16:04:24 -0700 (Thu, 16 Mar 2006) $
  * @see JDOManager#getDatabase()
  * @see Query
  */
@@ -109,14 +109,6 @@ public interface Database {
      *  the database when the transaction commits. */
     AccessMode READONLY = AccessMode.ReadOnly;
 
-    /** Read only access. Used with queries and the {@link #load(Class,Object)} method to load
-     *  objects as read-only.
-     *  <br>
-     *  Read-only objects are not persistent and changes to these objects are not reflected in
-     *  the database when the transaction commits.
-     *  @deprecated Use READONLY instead. */
-    AccessMode ReadOnly = AccessMode.ReadOnly;
-
     /** Shared access. Used with queries and the {@link #load(Class,Object)} method to load
      *  objects with shared access.
      *  <br/>
@@ -128,18 +120,6 @@ public interface Database {
      *  as such, and a cached copy is used to populate the object. */
     AccessMode SHARED = AccessMode.Shared;
 
-    /** Shared access. Used with queries and the {@link #load(Class,Object)} method to load
-     *  objects with shared access.
-     *  <br/>
-     *  Shared access allows the same record to be accessed by two concurrent transactions, each
-     *  with it's own view (object).
-     *  <br/>
-     *  These objects acquire a read lock which escalated to a write lock when the transaction
-     *  commits if the object has been modified. Dirty checking is enabled for all fields marked
-     *  as such, and a cached copy is used to populate the object.
-     *  @deprecated Use SHARED instead. */
-    AccessMode Shared = AccessMode.Shared;
-
     /** Exclusive access. Used with queries and the {@link #load(Class,Object)} method to load
      *  objects with exclusive access.
      *  <br/>
@@ -152,19 +132,6 @@ public interface Database {
      *  cache. */
     AccessMode EXCLUSIVE = AccessMode.Exclusive;
 
-    /** Exclusive access. Used with queries and the {@link #load(Class,Object)} method to load
-     *  objects with exclusive access.
-     *  <br/>
-     *  Exclusive access prevents two concurrent transactions from accessing the same record. In
-     *  exclusive mode objects acquire a write lock, and concurrent transactions will block until
-     *  the lock is released at commit time.
-     *  <br/>
-     *  Dirty checking is enabled for all fields marked as such. When an object is first loaded
-     *  in the transaction, it will be synchronized with the database and not populated from the
-     *  cache.
-     *  @deprecated Use EXCLUSIVE instead. */
-    AccessMode Exclusive = AccessMode.Exclusive;
-
     /** Database lock access. Used with queries and the {@link #load(Class,Object)} method to
      *  load objects with a database lock.
      *  <br/>
@@ -176,18 +143,6 @@ public interface Database {
      *  database and not populated from the cache. Dirty checking is not required. */
     AccessMode DBLOCKED = AccessMode.DbLocked;
 
-    /** Database lock access. Used with queries and the {@link #load(Class,Object)} method to
-     *  load objects with a database lock.
-     *  <br/>
-     *  Database lock prevents two concurrent transactions from accessing the same record either
-     *  through Castor or direct database access by acquiring a write lock in the select statement.
-     *  Concurrent transactions will block until the lock is released at commit time.
-     *  <br/>
-     *  When an object is first loaded in the transaction, it will be synchronized with the
-     *  database and not populated from the cache. Dirty checking is not required.
-     *  @deprecated Use DBLOCKED instead. */
-    AccessMode DbLocked = AccessMode.DbLocked;
-
     /**
      * Creates an OQL query with no statement. {@link OQLQuery#create}
      * must be called before the query can be executed.
@@ -429,6 +384,9 @@ public interface Database {
      * transaction. If it is turn off, only dependent object will 
      * be created automatically.
      * @return True if the current transaction is set to 'autoStore'.
+     * 
+     * @deprecated As of Castor 1.3.2, please use the 'cascading' attribute 
+     *         of the field mapping instead.
      */
     boolean isAutoStore();
 
@@ -443,7 +401,7 @@ public interface Database {
      * theDataObject.
      * <p>
      * If db.update( theDataObject ), and theDataObject is
-     * loaded/queuied/created in a previous transaction, Castor
+     * loaded/queried/created in a previous transaction, Castor
      * will let theDataObject, and all reachable object from
      * theDataObject, participate in the current transaction.
      * <p>
@@ -451,6 +409,9 @@ public interface Database {
      * dependent object, and related objects must be created/update 
      * explicitly.
      * @param autoStore True if this feature should be enabled.
+     * 
+     * @deprecated As of Castor 1.3.2, please use the 'cascading' attribute 
+     *         of the field mapping instead.
      */
     void setAutoStore(boolean autoStore);
 
