@@ -40,11 +40,13 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: ClassMolder.java 8427 2009-09-06 13:16:28Z lukaslang $
+ * $Id: ClassMolder.java 8749 2010-09-16 08:50:29Z wguttmn $
  */
 package org.exolab.castor.persist;
 
 import java.sql.Connection;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.Properties;
 import java.util.Vector;
 
@@ -73,10 +75,10 @@ import org.exolab.castor.mapping.FieldDescriptor;
 import org.exolab.castor.mapping.FieldHandler;
 import org.exolab.castor.mapping.MappingException;
 import org.exolab.castor.mapping.TypeConvertor;
+import org.exolab.castor.mapping.loader.ClassDescriptorHelper;
 import org.exolab.castor.mapping.loader.ClassDescriptorImpl;
 import org.exolab.castor.mapping.loader.FieldHandlerImpl;
 import org.exolab.castor.mapping.xml.ClassMapping;
-import org.exolab.castor.mapping.xml.FieldMapping;
 import org.exolab.castor.mapping.xml.NamedNativeQuery;
 import org.exolab.castor.persist.spi.CallbackInterceptor;
 import org.exolab.castor.persist.spi.Identity;
@@ -104,7 +106,7 @@ import org.exolab.castor.xml.ResolverException;
  *
  * @author <a href="yip@intalio.com">Thomas Yip</a>
  * @author <a href="mailto:ferret AT frii dot com">Bruce Snyder</a>
- * @author <a href="mailto:wernert DOT guttmann AT gmx DOT net">Werner Guttmann</a>
+ * @author <a href="mailto:werner DOT guttmann AT gmx DOT net">Werner Guttmann</a>
  */
 
 public class ClassMolder {
@@ -158,9 +160,6 @@ public class ClassMolder {
     /** Is a key kenerator used for the base class? */
     public boolean _isKeyGenUsed;
 
-    /** True if org.exolab.castor.debug="true". */
-    private boolean _debug;
-
     /** Create priority. */
     private int _priority = -1;
 
@@ -178,178 +177,176 @@ public class ClassMolder {
      * 
      * @param ds is the helper class for resolving depends and extends relationship
      *        among all the ClassMolder in the same LockEngine.
-     * @param classDescrResolver {@link ClassDescriptorResolver} instance
+     * @param classDescriptorResolver {@link ClassDescriptorResolver} instance
      * @param lock the lock engine.
-     * @param clsDesc the classDescriptor for the base class.
-     * @param persist the Persistence for the base class.
+     * @param classDescriptor the classDescriptor for the base class.
+     * @param persistenceEngine the Persistence for the base class.
      * @throws ClassNotFoundException If a class cannot be loaded.
      * @throws MappingException if an error occurred with analyzing the mapping information.
      */
-    ClassMolder(final DatingService ds, final ClassDescriptorResolver classDescrResolver, 
-            final LockEngine lock,
-            final ClassDescriptor clsDesc, final Persistence persist)
+    ClassMolder(final DatingService ds, final ClassDescriptorResolver classDescriptorResolver, 
+            final LockEngine lock, final ClassDescriptor classDescriptor, 
+            final Persistence persistenceEngine)
             throws ClassNotFoundException, MappingException {
 
-        _debug = Boolean.getBoolean("org.exolab.castor.debug");
         _engine = lock;
-        _persistence = persist;
-        _clsDesc = clsDesc;
+        _persistence = persistenceEngine;
+        _clsDesc = classDescriptor;
         
-        ClassMapping clsMap = ((ClassDescriptorImpl) clsDesc).getMapping();
-        _name = clsMap.getName();
-        _accessMode = AccessMode.valueOf(clsMap.getAccess().toString());
-
+        _name = classDescriptor.getJavaClass().getName();
         ds.register(_name, this);
-
-        ClassMapping dependClassMapping = (ClassMapping) clsMap.getDepends();
-        ClassMapping extendsClassMapping = (ClassMapping) clsMap.getExtends();
-
-        //if ( dep != null && ext != null )
-        //    throw new MappingException("A JDO cannot both extends and depends on other objects");
-
-        if (dependClassMapping != null) {
-            ds.pairDepends(this, dependClassMapping.getName());
+        
+        if (classDescriptor.hasNature(ClassDescriptorJDONature.class.getName())) {
+            ClassDescriptorJDONature nature = new ClassDescriptorJDONature(classDescriptor);
+            _accessMode = nature.getAccessMode();
         }
+        
+        dealWithExtendsAndDepends(ds, classDescriptor);
 
-        if (extendsClassMapping != null) {
-            ds.pairExtends(this, extendsClassMapping.getName());
-        }
-
-        if (clsDesc.hasNature(ClassDescriptorJDONature.class.getName())) {
-            ClassDescriptorJDONature nature;
-            nature = new ClassDescriptorJDONature(clsDesc);
+        if (classDescriptor.hasNature(ClassDescriptorJDONature.class.getName())) {
+            ClassDescriptorJDONature nature = new ClassDescriptorJDONature(classDescriptor);
             _cacheParams = nature.getCacheParams();
             _isKeyGenUsed = nature.getKeyGeneratorDescriptor() != null;
         }
 
         // construct <tt>FieldMolder</tt>s for each of the identity fields of
         // the base class.
-        FieldMapping[] fmId = ClassMolderHelper.getIdFields(clsMap);
-        _ids = new FieldMolder[fmId.length];
-        for (int i = 0; i < _ids.length; i++) {
-            _ids[i] = new FieldMolder(ds, this, fmId[i]);
+        
+        FieldDescriptor[] identityDescriptors = ClassDescriptorHelper.getIdFields(classDescriptor);
+        
+        _ids = new FieldMolder[identityDescriptors.length];
+        int m = 0;
+        for (FieldDescriptor identityDescriptor : identityDescriptors) {
+            _ids[m++] = new FieldMolder(ds, this, identityDescriptor);
         }
 
         // construct <tt>FieldModlers</tt>s for each of the non-transient fields 
         // of the base class 
-        FieldMapping[] fmFields = ClassMolderHelper.getFullFields(clsMap);
+        
+        FieldDescriptor[] fieldDescriptors = ClassDescriptorHelper.getFullFields(classDescriptor);
         
         int numberOfNonTransientFieldMolders = 0;
-        for (int i = 0; i < fmFields.length; i++) {
-            if (!isFieldTransient(fmFields[i])) {
+        for (FieldDescriptor fieldDescriptor : fieldDescriptors) {
+            if (!isFieldTransient(fieldDescriptor)) {
                 numberOfNonTransientFieldMolders += 1;
             }
         }
+
         _fhs = new FieldMolder[numberOfNonTransientFieldMolders];
         _resolvers = new ResolverStrategy[numberOfNonTransientFieldMolders];
         
-        int fieldMolderNumber = 0;
-        for (int i = 0; i < fmFields.length; i++) {
-            
+        int fieldMolderCount = 0;
+        for (FieldDescriptor fieldDescriptor : fieldDescriptors) {
+
+
             // don't create field molder for transient fields
-            if (isFieldTransient(fmFields[i])) {
+            if (isFieldTransient(fieldDescriptor)) {
                 continue;
             }
-            
-            if ((fmFields[i].getSql() != null) && (fmFields[i].getSql().getManyTable() != null)) {
+
+            if (fieldDescriptor.hasNature(FieldDescriptorJDONature.class.getName()) && 
+                    new FieldDescriptorJDONature(fieldDescriptor).getManyTable() != null) {
+                FieldDescriptorJDONature nature = new FieldDescriptorJDONature(fieldDescriptor);
+                
                 // the fields is not primitive
                 String[] relatedIdSQL = null;
                 int[] relatedIdType = null;
                 TypeConvertor[] relatedIdConvertTo = null;
                 TypeConvertor[] relatedIdConvertFrom = null;
-                
-                String manyTable = fmFields[i].getSql().getManyTable();
 
-                String[] idSQL = new String[fmId.length];
-                int[] idType = new int[fmId.length];
-                TypeConvertor[] idConvertFrom = new TypeConvertor[fmId.length];
-                TypeConvertor[] idConvertTo = new TypeConvertor[fmId.length];
-                FieldDescriptor[] fd = ((ClassDescriptorImpl) clsDesc).getIdentities();
-                for (int j = 0; j < fmId.length; j++) {
-                    idSQL[j] = fmId[j].getSql().getName()[0];
+                String manyTable = nature.getManyTable();
 
-                    if (fd[j].hasNature(FieldDescriptorJDONature.class.getName())) {
-                        int[] type = new FieldDescriptorJDONature(fd[j]).getSQLType();
-                        idType[j] = (type == null) ? 0 : type[0];
-                        FieldHandlerImpl fh = (FieldHandlerImpl) fd[j].getHandler();
-                        idConvertTo[j] = fh.getConvertTo();
-                        idConvertFrom[j] = fh.getConvertFrom();
+                String[] idSQL = new String[identityDescriptors.length];
+                int[] idType = new int[identityDescriptors.length];
+                TypeConvertor[] idConvertFrom = new TypeConvertor[identityDescriptors.length];
+                TypeConvertor[] idConvertTo = new TypeConvertor[identityDescriptors.length];
+                FieldDescriptor[] identityFieldDescriptors = ((ClassDescriptorImpl) classDescriptor).getIdentities();
+                int identityFieldCount = 0;
+                for (FieldDescriptor identityFieldDescriptor : identityFieldDescriptors) {
+                    if (identityFieldDescriptor.hasNature(FieldDescriptorJDONature.class.getName())) {
+                        idSQL[identityFieldCount] = new FieldDescriptorJDONature(identityFieldDescriptor).getSQLName()[0];
+                        int[] type = new FieldDescriptorJDONature(identityFieldDescriptor).getSQLType();
+                        idType[identityFieldCount] = (type == null) ? 0 : type[0];
+                        FieldHandlerImpl fieldHandler = (FieldHandlerImpl) identityFieldDescriptor.getHandler();
+                        idConvertTo[identityFieldCount] = fieldHandler.getConvertTo();
+                        idConvertFrom[identityFieldCount] = fieldHandler.getConvertFrom();
                     } else {
                         throw new MappingException(
                                 "Identity type must contains sql information: " + _name);
                     }
+                    identityFieldCount++;
                 }
 
-                ClassDescriptor relDesc = null;
+                ClassDescriptor relatedClassDescriptor = null;
                 try {
-                    JDOClassDescriptorResolver jdoCDR;
-                    jdoCDR = (JDOClassDescriptorResolver) classDescrResolver;
-                    relDesc = jdoCDR.resolve(fmFields[i].getType());
+                    JDOClassDescriptorResolver jdoCDR = (JDOClassDescriptorResolver) classDescriptorResolver;
+                    relatedClassDescriptor = jdoCDR.resolve(fieldDescriptor.getFieldType().getName());
                 } catch (ResolverException e) {
                     throw new MappingException("Problem resolving class descriptor for class " 
-                            + fmFields.getClass(), e);
+                            + fieldDescriptor.getClass().getName(), e);
                 }
-                                
-                if (relDesc.hasNature(ClassDescriptorJDONature.class.getName())) {
-                    FieldDescriptor[] relatedIds = ((ClassDescriptorImpl) relDesc).getIdentities();
-                    relatedIdSQL = new String[relatedIds.length];
-                    relatedIdType = new int[relatedIds.length];
-                    relatedIdConvertTo = new TypeConvertor[relatedIds.length];
-                    relatedIdConvertFrom = new TypeConvertor[relatedIds.length];
-                    for (int j = 0; j < relatedIdSQL.length; j++) {
-                        if (relatedIds[j].hasNature(FieldDescriptorJDONature.class.getName())) {
-                            FieldDescriptorJDONature nature;
-                            nature = new FieldDescriptorJDONature(relatedIds[j]);
-                            String[] tempId = nature.getSQLName();
-                            relatedIdSQL[j] = (tempId == null) ? null : tempId[0];
-                            int[] tempType =  nature.getSQLType();
-                            relatedIdType[j] = (tempType == null) ? 0 : tempType[0];
-                            FieldHandlerImpl fh = (FieldHandlerImpl) relatedIds[j].getHandler();
-                            relatedIdConvertTo[j] = fh.getConvertTo();
-                            relatedIdConvertFrom[j] = fh.getConvertFrom();
+
+                if (relatedClassDescriptor.hasNature(ClassDescriptorJDONature.class.getName())) {
+                    FieldDescriptor[] relatedIdentityDescriptors = ((ClassDescriptorImpl) relatedClassDescriptor).getIdentities();
+                    relatedIdSQL = new String[relatedIdentityDescriptors.length];
+                    relatedIdType = new int[relatedIdentityDescriptors.length];
+                    relatedIdConvertTo = new TypeConvertor[relatedIdentityDescriptors.length];
+                    relatedIdConvertFrom = new TypeConvertor[relatedIdentityDescriptors.length];
+                    int relatedIdentityCount = 0;
+                    for (FieldDescriptor relatedIdentityDescriptor : relatedIdentityDescriptors) {
+                        if (relatedIdentityDescriptor.hasNature(FieldDescriptorJDONature.class.getName())) {
+                            FieldDescriptorJDONature relatedNature = new FieldDescriptorJDONature(relatedIdentityDescriptor);
+                            String[] tempId = relatedNature.getSQLName();
+                            relatedIdSQL[relatedIdentityCount] = (tempId == null) ? null : tempId[0];
+                            int[] tempType =  relatedNature.getSQLType();
+                            relatedIdType[relatedIdentityCount] = (tempType == null) ? 0 : tempType[0];
+                            FieldHandlerImpl fh = (FieldHandlerImpl) relatedIdentityDescriptors[relatedIdentityCount].getHandler();
+                            relatedIdConvertTo[relatedIdentityCount] = fh.getConvertTo();
+                            relatedIdConvertFrom[relatedIdentityCount] = fh.getConvertFrom();
                         } else {
                             throw new MappingException(
                                     "Field type is not persistence-capable: "
-                                    + relatedIds[j].getFieldName());
+                                    + relatedIdentityDescriptors[relatedIdentityCount].getFieldName());
                         }
+                        relatedIdentityCount++;
                     }
                 }
 
                 // if many-key exist, idSQL is overridden
-                String[] manyKey = fmFields[i].getSql().getManyKey();
+                String[] manyKey = nature.getManyKey();
                 if ((manyKey != null) && (manyKey.length != 0)) {
                     if (manyKey.length != idSQL.length) {
                         throw new MappingException(
                                 "The number of many-keys doesn't match referred object: "
-                                + clsDesc.getJavaClass().getName());
+                                + classDescriptor.getJavaClass().getName());
                     }
                     idSQL = manyKey;
                 }
 
                 // if name="" exist, relatedIdSQL is overridden
-                String[] manyName = fmFields[i].getSql().getName();
+                String[] manyName = nature.getSQLName();
                 if ((manyName != null) && (manyName.length != 0)) {
                     if (manyName.length != relatedIdSQL.length) {
                         throw new MappingException(
                                 "The number of many-keys doesn't match referred object: "
-                                + relDesc.getJavaClass().getName());
+                                + relatedClassDescriptor.getJavaClass().getName());
                     }
                     relatedIdSQL = manyName;
                 }
-                
+
                 SQLRelationLoader loader = _persistence.createSQLRelationLoader(
                         manyTable, idSQL, idType, idConvertTo, idConvertFrom,
                         relatedIdSQL, relatedIdType, relatedIdConvertTo, relatedIdConvertFrom);
-                _fhs[fieldMolderNumber] = new FieldMolder(ds, this, fmFields[i], loader);
+                _fhs[fieldMolderCount] = new FieldMolder(ds, this, fieldDescriptor, loader);
             } else {
-                _fhs[fieldMolderNumber] = new FieldMolder(ds, this, fmFields[i]);
+                _fhs[fieldMolderCount] = new FieldMolder(ds, this, fieldDescriptor);
             }
-            
-            // create RelationResolver instance
-            _resolvers[fieldMolderNumber] = ResolverFactory.createRelationResolver(
-                    _fhs[fieldMolderNumber], this, fieldMolderNumber, _debug);
 
-            fieldMolderNumber += 1;
+            // create RelationResolver instance
+            _resolvers[fieldMolderCount] = ResolverFactory.createRelationResolver(
+                    _fhs[fieldMolderCount], this, fieldMolderCount);
+
+            fieldMolderCount += 1;
         }
 
         // ssa, FIXME : Are the two statements equivalents ?
@@ -358,13 +355,43 @@ public class ClassMolder {
             _callback = new JDOCallback();
         }
     }
-    
-    public ClassDescriptor getClassDescriptor() { return _clsDesc; }
 
-    private boolean isFieldTransient(final FieldMapping fieldMapping) {
-        boolean isFieldTransient = fieldMapping.getTransient();
-        if (fieldMapping.getSql() != null) {
-            isFieldTransient |= fieldMapping.getSql().getTransient();
+    /**
+     * Remaining method that relies on the usage of {@link ClassMapping} to extract
+     * information from the JDO mapping file rather than relying on {@link ClassDescriptor}
+     * and associated JDO-specific natures.
+     * 
+     * @param ds {@link DatingService} instance.
+     * @param classDescriptor The {@link ClassDescriptor} instance used to describe the class at hand.
+     * @throws MappingException If something unforeseen happens ....
+     */
+    private void dealWithExtendsAndDepends(final DatingService ds,
+            final ClassDescriptor classDescriptor) throws MappingException {
+
+        ClassDescriptor dependClassDescriptor = ((ClassDescriptorImpl) classDescriptor).getDepends();
+        ClassDescriptor extendsClassDescriptor = ((ClassDescriptorImpl) classDescriptor).getExtends();
+
+        //if ( dep != null && ext != null )
+        //    throw new MappingException("A JDO cannot both extends and depends on other objects");
+
+        if (dependClassDescriptor != null) {
+            ds.pairDepends(this, dependClassDescriptor.getJavaClass().getName());
+        }
+
+        if (extendsClassDescriptor != null) {
+            ds.pairExtends(this, extendsClassDescriptor.getJavaClass().getName());
+        }
+    }
+    
+    public ClassDescriptor getClassDescriptor() { 
+        return _clsDesc; 
+    }
+    
+    private boolean isFieldTransient(final FieldDescriptor fieldDescriptor) {
+        boolean isFieldTransient = fieldDescriptor.isTransient();
+        if (fieldDescriptor.hasNature(FieldDescriptorJDONature.class.getName())) {
+            FieldDescriptorJDONature nature = new FieldDescriptorJDONature(fieldDescriptor);
+            isFieldTransient |= nature.isTransient();
         }
         return isFieldTransient;
     }
@@ -377,10 +404,10 @@ public class ClassMolder {
      * be set null. If the related object is a Collection, then
      * the related object will be removed from the Collection. <p>
      *
-     * If any changed occured, transactionContext.markModified
+     * If any changed occurred, transactionContext.markModified
      * will be called, to indicate the object is modified. <p>
      *
-     * It method will iterate thur all of the object's field and
+     * It method will iterate through all of the object's field and
      * try to remove all the occurrence.
      *
      * @param tx the TransactionContext of the transaction in action
@@ -396,8 +423,6 @@ public class ClassMolder {
         boolean updateCache = false;
         boolean updatePersist = false;
         
-        resetResolvers();
-        
         UpdateAndRemovedFlags flags = null;
         for (int i = 0; i < _fhs.length; i++) {
             flags = _resolvers[i].removeRelation(tx, object, relatedMolder,
@@ -426,33 +451,50 @@ public class ClassMolder {
             // circular reference detected, do not loop
             return 0;
         } else if (_priority < 0) {
-            // mark current Molder to avoid infinite loop with circular reference
-            _priority = -2;
-            int maxPrior = 0;
-            for (int i = 0; i < _fhs.length; i++) {
-                if (_fhs[i].isPersistanceCapable()
-                        && (_fhs[i].getFieldClassMolder() != this)
-                        && _fhs[i].isStored()) {
-                    int refPrior = _fhs[i].getFieldClassMolder().getPriority() + 1;
-                    maxPrior = Math.max(maxPrior, refPrior);
+            // find root of extends hierarchy
+            ClassMolder root = this;
+            while (root._extends != null) { root = root._extends; }
+            
+            // find all class molders of extends hierarchy
+            List<ClassMolder> molders = getAllExtendentMolders(root);
+            
+            // mark all class molders of extends hierarchy as work in progress
+            for (ClassMolder molder : molders) { molder._priority = -2; }
+            
+            // preset maximum priority
+            int maxPriority = 0;
+            
+            // calculate maximum priority from all classes referenced by the whole hierarchy
+            for (ClassMolder molder : molders) {
+                FieldMolder[] fhs = molder._fhs;
+                for (int i = 0; i < fhs.length; i++) {
+                    FieldMolder fh = fhs[i];
+                    if (fh.isPersistanceCapable() && fh.isStored()
+                            && (fh.getFieldClassMolder() != this)) {
+                        int refPriority = fh.getFieldClassMolder().getPriority() + 1;
+                        maxPriority = Math.max(maxPriority, refPriority);
+                    }
                 }
-                /* should and "if case" for add _ids[i].isForeginKey() in the future */
             }
 
-            /* adjust priority if there are dependent classes */
-            if (_depends != null) {
-                maxPrior = Math.max(maxPrior, _depends.getPriority() + 1);
-            }
-
-            /* adjust priority if there are extendent classes */
-            if (_extends != null) {
-                  maxPrior = Math.max(maxPrior, _extends.getPriority() + 1);
-            }
-
-            _priority  = maxPrior;
+            // set priority of the whole hierarchy to the calculated maximum
+            for (ClassMolder molder : molders) { molder._priority = maxPriority; }
         }
+        
         return _priority;
     }
+    
+    private List<ClassMolder> getAllExtendentMolders(final ClassMolder root) {
+        List<ClassMolder> molders = new ArrayList<ClassMolder>();
+        molders.add(root);
+        
+        if (root._extendent != null) {
+            for (ClassMolder extendent : root._extendent) {
+                molders.addAll(getAllExtendentMolders(extendent));
+            }
+        }
+        return molders;
+    }
 
     public void loadTimeStamp(final TransactionContext tx, final DepositBox locker,
             final AccessMode suggestedAccessMode)
@@ -533,23 +575,19 @@ public class ClassMolder {
     throws PersistenceException {
         OID oid = locker.getOID();
 
-        resetResolvers();
-
         // Check for version field.
         if (_clsDesc.hasNature(ClassDescriptorJDONature.class.getName())) {
-            ClassDescriptorJDONature jdoNature =
-                    new ClassDescriptorJDONature(_clsDesc);
+            ClassDescriptorJDONature jdoNature = new ClassDescriptorJDONature(_clsDesc);
             String versionField = jdoNature.getVersionField();
 
             // Check if version field was set and has content.
+            // TODO: that check should be moved to e.g. ClassDescriptorJDONature
             if (versionField != null && versionField.length() > 0) {
                 // Find field descriptor for version field.
-                FieldDescriptor versionFieldDescriptor =
-                        jdoNature.getField(versionField);
+                FieldDescriptor versionFieldDescriptor = jdoNature.getField(versionField);
                 FieldHandler fieldHandler = versionFieldDescriptor.getHandler();
                 // Set the entity's version to the locker's version.
-                fieldHandler.setValue(proposedObject.getEntity(), locker
-                        .getVersion());
+                fieldHandler.setValue(proposedObject.getEntity(), locker.getVersion());
             }
         }
 
@@ -564,14 +602,14 @@ public class ClassMolder {
 
         // iterates over all the field of the object and bind all field.
         for (int i = 0; i < _fhs.length; i++) {
-            int fieldType = _fhs[i].getFieldType();
+            FieldPersistenceType fieldType = _fhs[i].getFieldPertsistenceType();
             
             switch (fieldType) {
-            case FieldMolder.PRIMITIVE:
-            case FieldMolder.SERIALIZABLE:
-            case FieldMolder.PERSISTANCECAPABLE:
-            case FieldMolder.ONE_TO_MANY:
-            case FieldMolder.MANY_TO_MANY:
+            case PRIMITIVE:
+            case SERIALIZABLE:
+            case PERSISTANCECAPABLE:
+            case ONE_TO_MANY:
+            case MANY_TO_MANY:
                 _resolvers[i].load(tx, oid, proposedObject, accessMode);
                 break;
             default:
@@ -616,10 +654,6 @@ public class ClassMolder {
     public Identity create(final TransactionContext tx, final OID oid, final DepositBox locker,
             final Object object) throws PersistenceException {
 
-        int fieldType;
-
-        resetResolvers();
-
         if (_persistence == null) {
             throw new PersistenceException("non persistence capable: "
                     + oid.getName());
@@ -633,11 +667,11 @@ public class ClassMolder {
 
         // Check for version field.
         if (_clsDesc.hasNature(ClassDescriptorJDONature.class.getName())) {
-            ClassDescriptorJDONature jdoNature = 
-                new ClassDescriptorJDONature(_clsDesc);
+            ClassDescriptorJDONature jdoNature = new ClassDescriptorJDONature(_clsDesc);
             String versionField = jdoNature.getVersionField();
 
             // Check if version field was set and has content.
+            // TODO: that check should be moved to e.g. ClassDescriptorJDONature
             if (versionField != null && versionField.length() > 0) {
                 // Find field descriptor for version field.
                 FieldDescriptor versionFieldDescriptor =
@@ -654,14 +688,14 @@ public class ClassMolder {
 
         // copy the object to cache should make a new field now,
         for (int i = 0; i < _fhs.length; i++) {
-            fieldType = _fhs[i].getFieldType();
+            FieldPersistenceType fieldPersistenceType = _fhs[i].getFieldPertsistenceType();
 
-            switch (fieldType) {
-            case FieldMolder.PRIMITIVE:
-            case FieldMolder.PERSISTANCECAPABLE:
-            case FieldMolder.SERIALIZABLE:
-            case FieldMolder.ONE_TO_MANY:
-            case FieldMolder.MANY_TO_MANY:
+            switch (fieldPersistenceType) {
+            case PRIMITIVE:
+            case PERSISTANCECAPABLE:
+            case SERIALIZABLE:
+            case ONE_TO_MANY:
+            case MANY_TO_MANY:
                 entity.setField(_resolvers[i].create(tx, object), i);
                 break;
 
@@ -687,6 +721,15 @@ public class ClassMolder {
 
         // after successful creation, add the entry in the relation table for
         // all many-to-many relationship
+        //ASE: This is the source of problem with M:N relations. As we see at
+        //     this point, for every persisted object in any M:N relation, a
+        //	   new entry to the relation table is created. But this happens not
+        //	   only when both related objects are persisted but also after
+        //	   persistence of the first. But that cannot work!
+        // 	   A solution would be to store a collection of relations which have
+        //	   to persisted in the relation table after all objects are persisted 
+        //	   independently!
+        
         for (int i = 0; i < _fhs.length; i++) {
             entity.setField(_resolvers[i].postCreate(
                     tx, oid, object, entity.getField(i), createdId), i);
@@ -709,8 +752,6 @@ public class ClassMolder {
 
         boolean updateCache = false;
 
-        resetResolvers();
-
         // iterate all the fields and mark all the dependent object.
         for (int i = 0; i < _fhs.length; i++) {
             updateCache |= _resolvers[i].markCreate(tx, oid, object);
@@ -819,11 +860,11 @@ public class ClassMolder {
 
         // Set the new timestamp into version field.
         if (_clsDesc.hasNature(ClassDescriptorJDONature.class.getName())) {
-            ClassDescriptorJDONature jdoNature =
-                    new ClassDescriptorJDONature(_clsDesc);
+            ClassDescriptorJDONature jdoNature = new ClassDescriptorJDONature(_clsDesc);
             String versionField = jdoNature.getVersionField();
 
             // Check if version field was set and has content.
+            // TODO: that check should be moved to e.g. ClassDescriptorJDONature
             if (versionField != null && versionField.length() > 0) {
                 // Find field descriptor for version field.
                 FieldDescriptor versionFieldDescriptor =
@@ -889,8 +930,6 @@ public class ClassMolder {
 
         AccessMode accessMode = getAccessMode(suggestedAccessMode);
 
-        resetResolvers();
-
         Object[] fields = locker.getObject(tx);
 
         boolean timeStampable = false;
@@ -902,16 +941,14 @@ public class ClassMolder {
         }
         
         ClassDescriptorJDONature jdoNature = null;
-        String versionField = null;
 
         // Check for version field.
         if (_clsDesc.hasNature(ClassDescriptorJDONature.class.getName())) {
-            // Initialize nature.
             jdoNature = new ClassDescriptorJDONature(_clsDesc);
-            versionField = jdoNature.getVersionField();
+            String versionField = jdoNature.getVersionField();
+            // TODO: that check should be moved to e.g. ClassDescriptorJDONature
             if (versionField != null && versionField.length() > 0) {
-                objectTimestamp =
-                        getObjectVersion(versionField, jdoNature, object);
+                objectTimestamp =  getObjectVersion(versionField, jdoNature, object);
                 timeStampable = true;
             }
         }
@@ -1006,7 +1043,6 @@ public class ClassMolder {
      *            the object.
      * @return the object's version.
      */
-    @SuppressWarnings("unchecked")
     private Long getObjectVersion(final String versionField,
             final ClassDescriptorJDONature jdoNature, final Object object) {
         FieldDescriptor versionFieldDescriptor =
@@ -1029,9 +1065,6 @@ public class ClassMolder {
             final Object object) {
 
         Object[] fields;
-        int fieldType;
-
-        resetResolvers();
 
         if (oid.getIdentity() == null) {
             throw new IllegalStateException(
@@ -1041,32 +1074,29 @@ public class ClassMolder {
         fields = new Object[_fhs.length];
 
         for (int i = 0; i < _fhs.length; i++) {
-            fieldType = _fhs[i].getFieldType();
-            switch (fieldType) {
-            case FieldMolder.PRIMITIVE:
-            case FieldMolder.SERIALIZABLE:
-            case FieldMolder.PERSISTANCECAPABLE:
-            case FieldMolder.ONE_TO_MANY:
-            case FieldMolder.MANY_TO_MANY:
+            FieldPersistenceType fieldPersistenceType = _fhs[i].getFieldPertsistenceType();
+            switch (fieldPersistenceType) {
+            case PRIMITIVE:
+            case SERIALIZABLE:
+            case PERSISTANCECAPABLE:
+            case ONE_TO_MANY:
+            case MANY_TO_MANY:
                 fields[i] = _resolvers[i].updateCache(tx, oid, object);
                 break;
             default:
                 throw new IllegalArgumentException("Field type invalid!");
             }
         }
-
         
-        ClassDescriptorJDONature jdoNature;
-        String versionField;
         Long objectVersion = null;
+        
         // Check for version field.
         if (_clsDesc.hasNature(ClassDescriptorJDONature.class.getName())) {
-            // Initialize nature.
-            jdoNature = new ClassDescriptorJDONature(_clsDesc);
-            versionField = jdoNature.getVersionField();
+            ClassDescriptorJDONature jdoNature = new ClassDescriptorJDONature(_clsDesc);
+            String versionField = jdoNature.getVersionField();
+            // TODO: that check should be moved to e.g. ClassDescriptorJDONature
             if (versionField != null && versionField.length() > 0) {
-                objectVersion =
-                        (Long) getObjectVersion(versionField, jdoNature, object);
+                objectVersion = getObjectVersion(versionField, jdoNature, object);
             }
         }
         
@@ -1089,9 +1119,6 @@ public class ClassMolder {
      * @param oid - the object identity of the target object
      */
     public void delete(final TransactionContext tx, final OID oid) throws PersistenceException {
-
-        resetResolvers();
-
         Connection conn = tx.getConnection(oid.getMolder().getLockEngine());
         Identity ids = oid.getIdentity();
 
@@ -1138,18 +1165,16 @@ public class ClassMolder {
     public void markDelete(final TransactionContext tx, final OID oid, final DepositBox locker,
             final Object object) throws PersistenceException {
 
-        resetResolvers();
-
         Object[] fields = locker.getObject(tx);
 
         for (int i = 0; i < _fhs.length; i++) {
-            int fieldType = _fhs[i].getFieldType();
+            FieldPersistenceType fieldType = _fhs[i].getFieldPertsistenceType();
             switch (fieldType) {
-            case FieldMolder.PRIMITIVE:
-            case FieldMolder.SERIALIZABLE:
-            case FieldMolder.PERSISTANCECAPABLE:
-            case FieldMolder.ONE_TO_MANY:
-            case FieldMolder.MANY_TO_MANY:
+            case PRIMITIVE:
+            case SERIALIZABLE:
+            case PERSISTANCECAPABLE:
+            case ONE_TO_MANY:
+            case MANY_TO_MANY:
                 _resolvers[i].markDelete(tx, object, fields[i]);
                 break;
             default:
@@ -1174,8 +1199,6 @@ public class ClassMolder {
             final Object object)
             throws PersistenceException {
 
-        resetResolvers();
-
         if (oid.getIdentity() == null) {
             throw new PersistenceException(
                     Messages.format("persist.missingIdentityForReverting", _name));
@@ -1206,7 +1229,7 @@ public class ClassMolder {
      */
     public Object newInstance(final ClassLoader loader)
     throws InstantiationException, IllegalAccessException, ClassNotFoundException {
-        Class aClass = null;
+        Class<?> aClass = null;
         aClass = ClassLoadingUtils.loadClass(loader, _name);
         return aClass.newInstance();
     }
@@ -1333,8 +1356,8 @@ public class ClassMolder {
      * @param loader the classloader.
      * @return the <code>Class</code> instance.
      */
-    public Class getJavaClass(final ClassLoader loader) {
-        Class result = null;
+    public Class<?> getJavaClass(final ClassLoader loader) {
+        Class<?> result = null;
         try {
             result = ClassLoadingUtils.loadClass(loader, _name);
         } catch (ClassNotFoundException e) {
@@ -1351,9 +1374,9 @@ public class ClassMolder {
      * @param cls the Class to check the assignation
      * @return true if assignable
      */
-    public boolean isAssignableFrom (final Class cls) {
+    public boolean isAssignableFrom (final Class<?> cls) {
         ClassLoader loader = cls.getClassLoader();
-        Class molderClass = null;
+        Class<?> molderClass = null;
         try {
             molderClass = ClassLoadingUtils.loadClass(loader, _name);
         } catch (ClassNotFoundException e) {
@@ -1485,7 +1508,7 @@ public class ClassMolder {
     public void resetResolvers () {
         if (!_resolversHaveBeenReset) {
             for (int i = 0; i < _fhs.length; i++) {
-                _resolvers[i] = ResolverFactory.createRelationResolver (_fhs[i], this, i, _debug);
+                _resolvers[i] = ResolverFactory.createRelationResolver (_fhs[i], this, i);
             }
             
             _resolversHaveBeenReset = true;
