@@ -15,7 +15,7 @@
  */
 package org.castor.cpa.persistence.sql.query.condition;
 
-import org.castor.cpa.persistence.sql.query.QueryContext;
+import org.castor.cpa.persistence.sql.query.Visitor;
 import org.castor.cpa.persistence.sql.query.expression.Expression;
 import org.junit.Ignore;
 
@@ -32,6 +32,8 @@ public final class PredicateMock extends Predicate {
         super(expression, evaluateTo);
     }
 
-    @Override
-    public void toString(final QueryContext ctx) { }
+    /**
+     * {@inheritDoc}
+     */
+    public void accept(final Visitor visitor) { }
 }
