@@ -15,7 +15,7 @@
  */
 package org.castor.cpa.persistence.sql.query.expression;
 
-import org.castor.cpa.persistence.sql.query.QueryContext;
+import org.castor.cpa.persistence.sql.query.Visitor;
 import org.junit.Ignore;
 
 /**
@@ -27,6 +27,8 @@ import org.junit.Ignore;
  */
 @Ignore
 public final class FunctionMock extends Expression {
-    @Override
-    public void toString(final QueryContext ctx) { }
+    /**
+     * {@inheritDoc}
+     */
+    public void accept(final Visitor visitor) { }
 }
