@@ -25,6 +25,7 @@ import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.castor.cpa.test.framework.CPATestCase;
 import org.castor.cpa.test.framework.xml.types.DatabaseEngineType;
+import org.castor.persist.proxy.LazyCollection;
 import org.exolab.castor.jdo.Database;
 import org.exolab.castor.jdo.DuplicateIdentityException;
 import org.exolab.castor.jdo.PersistenceException;
@@ -613,7 +614,7 @@ public final class TestLazyLoading extends CPATestCase {
         assertTrue(
                 "Collection has to be lazy! It is "
                         + loadPerson.getProjects().getClass(),
-                loadPerson.getProjects() instanceof org.exolab.castor.persist.Lazy);
+                loadPerson.getProjects() instanceof LazyCollection);
 
         // OK, the collection of projects is there, let's test a rollback for
         // bug #1046
@@ -624,7 +625,7 @@ public final class TestLazyLoading extends CPATestCase {
         assertTrue(
                 "Collection has to be lazy! It is "
                         + loadPerson.getProjects().getClass(),
-                loadPerson.getProjects() instanceof org.exolab.castor.persist.Lazy);
+                loadPerson.getProjects() instanceof LazyCollection);
     }
 
     public void testMasterUpdate() throws PersistenceException {
