@@ -79,6 +79,10 @@ public final class TestLoadLazy1toN extends TestCase {
         super.setUp();
 
         if (!_logHeader) {
+            LOG.info("");
+            LOG.info("");
+            LOG.info("TestLoadLazy1toN (" + (int) (10000 * TestCreate.FACTOR) + ")");
+            LOG.info("");
             LOG.info(format("", "begin", "result", "iterate", "commit", "close"));
             _logHeader = true;
         }
@@ -268,7 +272,7 @@ public final class TestLoadLazy1toN extends TestCase {
         int count = 0;
         while (results.hasMore()) {
             OID oid = (OID) results.next();
-            iterateStates((Locked) db.load(Locked.class, oid.getId()));
+            iterateStates(db.load(Locked.class, oid.getId()));
 
             count++;
         }
@@ -310,7 +314,7 @@ public final class TestLoadLazy1toN extends TestCase {
         int count = 0;
         while (results.hasMore()) {
             OID oid = (OID) results.next();
-            iterateStates((Locked) db.load(Locked.class, oid.getId()));
+            iterateStates(db.load(Locked.class, oid.getId()));
 
             count++;
         }
@@ -353,7 +357,7 @@ public final class TestLoadLazy1toN extends TestCase {
         int count = 0;
         while (results.hasMore()) {
             OID oid = (OID) results.next();
-            iterateStates((Locked) db.load(Locked.class, oid.getId(), Database.READONLY));
+            iterateStates(db.load(Locked.class, oid.getId(), Database.READONLY));
 
             count++;
         }
@@ -395,7 +399,7 @@ public final class TestLoadLazy1toN extends TestCase {
         int count = 0;
         while (results.hasMore()) {
             OID oid = (OID) results.next();
-            iterateStates((Locked) db.load(Locked.class, oid.getId(), Database.READONLY));
+            iterateStates(db.load(Locked.class, oid.getId(), Database.READONLY));
 
             count++;
         }
