@@ -40,21 +40,22 @@
  *
  * Copyright 1999-2003 (C) Intalio Inc. All Rights Reserved.
  *
- * $Id: ElementDecl.java 7372 2008-01-10 14:20:58Z wguttmn $
+ * $Id: ElementDecl.java 8667 2010-08-14 22:24:49Z wguttmn $
  */
 
 package org.exolab.castor.xml.schema;
 
-import org.exolab.castor.xml.*;
-import org.exolab.castor.xml.validators.ValidationUtils;
-
 import java.util.Enumeration;
+import java.util.Iterator;
 import java.util.Vector;
 
+import org.exolab.castor.xml.ValidationException;
+import org.exolab.castor.xml.validators.ValidationUtils;
+
 /**
  * An XML Schema ElementDecl
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 7372 $ $Date: 2006-04-25 15:08:23 -0600 (Tue, 25 Apr 2006) $
+ * @version $Revision: 8667 $ $Date: 2006-04-25 15:08:23 -0600 (Tue, 25 Apr 2006) $
 **/
 public class ElementDecl extends Particle implements Referable {
     /** SerialVersionUID */
@@ -441,9 +442,9 @@ public class ElementDecl extends Particle implements Referable {
      */
     public Enumeration getSubstitutionGroupMembers() {
     	Vector result = new Vector();
-    	Enumeration enumeration = _schema.getElementDecls();
-    	while (enumeration.hasMoreElements()) {
-    		ElementDecl temp  = (ElementDecl)enumeration.nextElement();
+    	Iterator<ElementDecl> enumeration = _schema.getElementDecls().iterator();
+    	while (enumeration.hasNext()) {
+    		ElementDecl temp  = (ElementDecl)enumeration.next();
     		String subName = temp.getSubstitutionGroup();
     		if (subName != null) {
                 // no namespace(s) or default namespace in use
