@@ -43,7 +43,7 @@
  * Portions of this file developed by Keith Visco after Jan 19 2005 are
  * Copyright (C) 2005 Keith Visco. All Rights Reserverd.
  *
- * $Id: AbstractMappingLoader.java 8054 2009-02-05 00:15:03Z rjoachim $
+ * $Id: AbstractMappingLoader.java 8813 2011-02-25 07:01:50Z wguttmn $
  */
 package org.exolab.castor.mapping.loader;
 
@@ -66,6 +66,7 @@ import org.castor.xml.InternalContext;
 import org.castor.xml.AbstractInternalContext;
 import org.castor.core.util.Messages;
 import org.exolab.castor.mapping.ClassDescriptor;
+import org.exolab.castor.mapping.ClonableFieldHandler;
 import org.exolab.castor.mapping.CollectionHandler;
 import org.exolab.castor.mapping.ConfigurableFieldHandler;
 import org.exolab.castor.mapping.ExtendedFieldHandler;
@@ -91,7 +92,7 @@ import org.exolab.castor.mapping.xml.Param;
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
  * @author <a href="keith AT kvisco DOT com">Keith Visco</a>
- * @version $Revision: 8054 $ $Date: 2006-04-10 16:39:24 -0600 (Mon, 10 Apr 2006) $
+ * @version $Revision: 8813 $ $Date: 2006-04-10 16:39:24 -0600 (Mon, 10 Apr 2006) $
  */
 public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
 
@@ -123,15 +124,18 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
     private static final String DELETE_METHOD_PREFIX = "delete";
 
     /** Empty array of class types used for reflection. */
-    protected static final Class[] EMPTY_ARGS = new Class[0];
+    protected static final Class<?>[] EMPTY_ARGS = new Class[0];
 
     /** The string argument for the valueOf method, used for introspection when searching for
      *  type-safe enumeration style classes. */
-    protected static final Class[] STRING_ARG = {String.class};
+    protected static final Class<?>[] STRING_ARG = {String.class};
 
     /** Factory method name for type-safe enumerations. */
     protected static final String VALUE_OF = "valueOf";
 
+    /** Method name to get string value of a type-safe enumerations. */
+    protected static final String NAME = "name";
+
     /**
      * The {@link AbstractInternalContext} is the centerpiece providing runtime configuration
      * and state information.
@@ -139,7 +143,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
     private InternalContext _internalContext;
 
     /** Map of field handlers associated by their name. */
-    private final Map _fieldHandlers = new HashMap();
+    private final Map<String, FieldHandler> _fieldHandlers = new HashMap<String, FieldHandler>();
 
     /**
      * Constructs a new mapping helper. This constructor is used by a derived class.
@@ -177,9 +181,9 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      */
     protected void createFieldHandlers(final MappingRoot mapping)
     throws MappingException {
-        Enumeration enumeration = mapping.enumerateFieldHandlerDef();
+        Enumeration<? extends FieldHandlerDef> enumeration = mapping.enumerateFieldHandlerDef();
         while (enumeration.hasMoreElements()) {
-            FieldHandlerDef def = (FieldHandlerDef) enumeration.nextElement();
+            FieldHandlerDef def = enumeration.nextElement();
             
             String name = def.getName();
             
@@ -188,7 +192,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
             }
             
             
-            Class clazz = resolveType(def.getClazz());
+            Class<?> clazz = resolveType(def.getClazz());
             FieldHandler fieldHandler = null;
             try {
                 if (!FieldHandler.class.isAssignableFrom(clazz)) {
@@ -218,9 +222,9 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
         
         // Gather the configuration data (parameters).
         Properties params = new Properties();            
-        Enumeration enumerateParam = def.enumerateParam();
+        Enumeration<? extends Param> enumerateParam = def.enumerateParam();
         while (enumerateParam.hasMoreElements()) {
-            Param par = (Param) enumerateParam.nextElement();
+            Param par = enumerateParam.nextElement();
             params.put(par.getName(), par.getValue());
         }
         
@@ -246,14 +250,16 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
     throws MappingException {
         // Load the mapping for all the classes. This is always returned
         // in the same order as it appeared in the mapping file.
-        Enumeration enumeration = mapping.enumerateClassMapping();
+        Enumeration<? extends ClassMapping> enumeration = mapping.enumerateClassMapping();
 
-        List retryList = new ArrayList();
+        List<ClassMapping> retryList = new ArrayList<ClassMapping>();
         while (enumeration.hasMoreElements()) {
-            ClassMapping clsMap = (ClassMapping) enumeration.nextElement();
+            ClassMapping clsMap = enumeration.nextElement();
             try {
                 ClassDescriptor clsDesc = createClassDescriptor(clsMap);
-                if (clsDesc != null) { addDescriptor(clsDesc); }
+                if (clsDesc != null) { 
+                    addDescriptor(clsDesc); 
+                }
             } catch (MappingException mx) {
                 // save for later for possible out-of-order mapping files...
                 retryList.add(clsMap);
@@ -263,15 +269,16 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
 
         // handle possible retries, for now we only loop once on the retries, but we
         // should change this to keep looping until we have no more success rate.
-        for (Iterator i = retryList.iterator(); i.hasNext();) {
-            ClassMapping clsMap = (ClassMapping) i.next();
-            ClassDescriptor clsDesc = createClassDescriptor(clsMap);
-            if (clsDesc != null) { addDescriptor(clsDesc); }
+        for (ClassMapping classMapping : retryList) {
+            ClassDescriptor clsDesc = createClassDescriptor(classMapping);
+            if (clsDesc != null) { 
+                addDescriptor(clsDesc); 
+            }
         }
 
         // iterate over all class descriptors and resolve relations between them
-        for (Iterator i = descriptorIterator(); i.hasNext();) {
-            resolveRelations((ClassDescriptor) i.next());
+        for (ClassDescriptor classDescriptor : getDescriptors()) {
+            resolveRelations(classDescriptor);
         }
     }
 
@@ -291,11 +298,13 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      *         ClassMapping but its descriptor could not be found.
      */
     protected final ClassDescriptor getExtended(final ClassMapping clsMap,
-                                                final Class javaClass) throws MappingException {
-        if (clsMap.getExtends() == null) { return null; }
+                                                final Class<?> javaClass) throws MappingException {
+        if (clsMap.getExtends() == null) { 
+            return null;
+        }
 
         ClassMapping mapping = (ClassMapping) clsMap.getExtends();
-        Class type = resolveType(mapping.getName());
+        Class<?> type = resolveType(mapping.getName());
         ClassDescriptor result = getDescriptor(type.getName());
 
         if (result == null) {
@@ -325,11 +334,13 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      *         ClassMapping but its descriptor could not be found.
      */
     protected final ClassDescriptor getDepended(final ClassMapping clsMap,
-                                                final Class javaClass) throws MappingException {
-        if (clsMap.getDepends() == null) { return null; }
+                                                final Class<?> javaClass) throws MappingException {
+        if (clsMap.getDepends() == null) { 
+            return null; 
+        }
 
         ClassMapping mapping = (ClassMapping) clsMap.getDepends();
-        Class type = resolveType(mapping.getName());
+        Class<?> type = resolveType(mapping.getName());
         ClassDescriptor result = getDescriptor(type.getName());
 
         if (result == null) {
@@ -349,7 +360,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * @throws MappingException If at least two fields have the same name.
      */
     protected final void checkFieldNameDuplicates(final FieldDescriptor[] fields,
-                                                  final Class cls) throws MappingException {
+                                                  final Class<?> cls) throws MappingException {
         for (int i = 0; i < fields.length - 1; i++) {
             String fieldName = fields[i].getFieldName();
             for (int j = i + 1; j < fields.length; j++) {
@@ -371,7 +382,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * <tt>java.lang.Integer</tt>). If the short name is used, the primitive type might
      * be returned.
      */
-    protected final Class resolveType(final String typeName)
+    protected final Class<?> resolveType(final String typeName)
     throws MappingException {
         try {
             return Types.typeFromName(getClassLoader(), typeName);
@@ -392,7 +403,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      *         be created.
      */
     protected final FieldDescriptorImpl[] createFieldDescriptors(final ClassMapping clsMap,
-                                                                     final Class javaClass) throws MappingException {
+                                                                     final Class<?> javaClass) throws MappingException {
         FieldMapping[] fldMap = null;
 
         if (clsMap.getClassChoice() != null) {
@@ -434,7 +445,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
 
     protected final FieldDescriptor[] divideFieldDescriptors(final FieldDescriptor[] fields,
             final String[] ids, final FieldDescriptor[] identities) {
-        List fieldList = new ArrayList(fields.length);
+        List<FieldDescriptor> fieldList = new ArrayList<FieldDescriptor>(fields.length);
 
         for (int i = 0; i < fields.length; i++) {
             FieldDescriptor field = fields[i];
@@ -456,7 +467,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
 
         // convert regularFieldList into array
         FieldDescriptor[] result = new FieldDescriptor[fieldList.size()];
-        return (FieldDescriptor[]) fieldList.toArray(result);
+        return fieldList.toArray(result);
     }
 
     /**
@@ -488,26 +499,26 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * @throws MappingException The field or its accessor methods are not
      *         found, not accessible, not of the specified type, etc.
      */
-    protected FieldDescriptorImpl createFieldDesc(final Class javaClass,
+    protected FieldDescriptorImpl createFieldDesc(final Class<?> javaClass,
                                                       final FieldMapping fieldMap) throws MappingException {
         String fieldName = fieldMap.getName();
 
         // If the field type is supplied, grab it and use it to locate the field/accessor.
-        Class fieldType = null;
+        Class<?> fieldType = null;
         if (fieldMap.getType() != null) {
             fieldType = resolveType(fieldMap.getType());
         }
 
         // If the field is declared as a collection, grab the collection type as
         // well and use it to locate the field/accessor.
-        CollectionHandler colHandler = null;
+        CollectionHandler collectionHandler = null;
         if (fieldMap.getCollection() != null) {
             String colTypeName = fieldMap.getCollection().toString();
-            Class colType = CollectionHandlers.getCollectionType(colTypeName);
-            colHandler = CollectionHandlers.getHandler(colType);
+            Class<?> collectionType = CollectionHandlers.getCollectionType(colTypeName);
+            collectionHandler = CollectionHandlers.getHandler(collectionType);
         }
 
-        TypeInfo typeInfo = getTypeInfo(fieldType, colHandler, fieldMap);
+        TypeInfo typeInfo = getTypeInfo(fieldType, collectionHandler, fieldMap);
 
         ExtendedFieldHandler exfHandler = null;
         FieldHandler handler = null;
@@ -525,10 +536,10 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
             // or a ClassCastException will be thrown... [KV 20030131 - also make sure
             // this new handler doesn't use it's own CollectionHandler otherwise it'll
             // cause unwanted calls to the getValue method during unmarshalling]
-            colHandler = typeInfo.getCollectionHandler();
+            collectionHandler = typeInfo.getCollectionHandler();
             typeInfo.setCollectionHandler(null);
             handler = new FieldHandlerImpl(handler, typeInfo);
-            typeInfo.setCollectionHandler(colHandler);
+            typeInfo.setCollectionHandler(collectionHandler);
             // End Castor JDO fix
         }
 
@@ -571,7 +582,22 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                 fieldName, typeInfo, handler, fieldMap.getTransient());
 
         fieldDesc.setRequired(fieldMap.getRequired());
-
+        
+        // set collection type as specified in mapping file
+        fieldDesc.setCollection(fieldMap.getCollection());
+        
+        // set various method informations 
+        fieldDesc.setComparator(fieldMap.getComparator());
+        fieldDesc.setCreateMethod(fieldMap.getCreateMethod());
+        fieldDesc.setGetMethod(fieldMap.getGetMethod());
+        fieldDesc.setSetMethod(fieldMap.getSetMethod());
+        
+        // set direct access (if defined)
+        fieldDesc.setDirect(fieldMap.getDirect());
+        
+        // extract values for 'laziness' from field mapping
+        fieldDesc.setLazy(fieldMap.isLazy());
+        
         // If we're using an ExtendedFieldHandler we need to set the FieldDescriptor
         if (exfHandler != null) {
             ((FieldHandlerFriend) exfHandler).setFieldDescriptor(fieldDesc);
@@ -585,12 +611,26 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
         
         // If there is a custom field handler present in the mapping, that one
         // is returned.
-        FieldHandler handler = (FieldHandler) _fieldHandlers.get(fieldMap.getHandler());
+        FieldHandler handler = _fieldHandlers.get(fieldMap.getHandler());
         if (handler != null) {
-        	return handler;
+
+            if (!(handler instanceof ClonableFieldHandler)) {
+                return handler;
+            }
+            
+            FieldHandler clonedHandler = handler;
+            Class<?> classToClone = handler.getClass();
+            try {
+                Method method = classToClone.getMethod("copyInstance", new Class[] { FieldHandler.class });
+                clonedHandler = (FieldHandler) method.invoke(handler, handler);
+                return clonedHandler;
+            } catch (Exception ex) {
+                String err = "The class '" + classToClone.getName() + "' must implement ClonableFieldHandler.";
+                throw new MappingException(err);
+            }
         }
         
-        Class handlerClass = null;
+        Class<?> handlerClass = null;
         handlerClass = resolveType(fieldMap.getHandler());
 
         if (!FieldHandler.class.isAssignableFrom(handlerClass)) {
@@ -602,7 +642,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
         // Get default constructor to invoke. We can't use the newInstance method
         // unfortunately becaue FieldHandler overloads this method
         try {
-            Constructor constructor = handlerClass.getConstructor(new Class[0]);
+            Constructor<?> constructor = handlerClass.getConstructor(new Class[0]);
             return (FieldHandler) constructor.newInstance(new Object[0]);
         } catch (Exception ex) {
             String err = "The class '" + handlerClass.getName()
@@ -617,9 +657,9 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * @param type Class to check for a public default constructor.
      * @return <code>true</code> if class has a public default constructor.
      */
-    private boolean hasPublicDefaultConstructor(final Class type) {
+    private boolean hasPublicDefaultConstructor(final Class<?> type) {
         try {
-            Constructor cons = type.getConstructor(EMPTY_ARGS);
+            Constructor<?> cons = type.getConstructor(EMPTY_ARGS);
             return Modifier.isPublic(cons.getModifiers());
         } catch (NoSuchMethodException ex) {
             return false;
@@ -633,13 +673,19 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * @return Static valueOf(String) factory method or <code>null</code> if none could
      *         be found.
      */
-    private Method getStaticValueOfMethod(final Class type) {
+    private Method getStaticValueOfMethod(final Class<?> type) {
         try {
             Method method = type.getMethod(VALUE_OF, STRING_ARG);
-            Class returnType = method.getReturnType();
-            if (returnType == null) { return null; }
-            if (!type.isAssignableFrom(returnType)) { return null; }
-            if (!Modifier.isStatic(method.getModifiers())) { return null; }
+            Class<?> returnType = method.getReturnType();
+            if (returnType == null) {
+                return null;
+            }
+            if (!type.isAssignableFrom(returnType)) { 
+                return null; 
+            }
+            if (!Modifier.isStatic(method.getModifiers())) { 
+                return null; 
+            }
             return method;
         } catch (NoSuchMethodException ex) {
             return null;
@@ -654,7 +700,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * @param fldMap the field mapping.
      * @return the newly created FieldHandler.
      */
-    protected final FieldHandler createFieldHandler(Class javaClass, Class fldType,
+    protected final FieldHandler createFieldHandler(Class<?> javaClass, Class<?> fldType,
                                                     final FieldMapping fldMap,
                                                     final TypeInfoReference typeInfoRef) throws MappingException {
         // Prevent introspection of transient fields
@@ -662,8 +708,8 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
             return new TransientFieldHandler();
         }
 
-        Class colType = null;
-        CollectionHandler colHandler = null;
+        Class<?> collectionType = null;
+        CollectionHandler collectionHandler = null;
         boolean colRequireGetSet = true;
 
         String fieldName = fldMap.getName();
@@ -672,17 +718,17 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
         // well and use it to locate the field/accessor.
         if (fldMap.getCollection() != null) {
             String colTypeName = fldMap.getCollection().toString();
-            colType = CollectionHandlers.getCollectionType(colTypeName);
-            colHandler = CollectionHandlers.getHandler(colType);
-            colRequireGetSet = CollectionHandlers.isGetSetCollection(colType);
-            if (colType == Object[].class) {
+            collectionType = CollectionHandlers.getCollectionType(colTypeName);
+            collectionHandler = CollectionHandlers.getHandler(collectionType);
+            colRequireGetSet = CollectionHandlers.isGetSetCollection(collectionType);
+            if (collectionType == Object[].class) {
                 if (fldType == null) {
                     String msg = "'type' is a required attribute for field that are "
                                + "array collections: " + fieldName;
                     throw new MappingException(msg);
                 }
                 Object obj = Array.newInstance(fldType, 0);
-                colType = obj.getClass();
+                collectionType = obj.getClass();
             }
         }
 
@@ -692,7 +738,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
         // If get/set methods not specified, use field names to determine them.
         if (fldMap.getDirect()) {
             // No accessor, map field directly.
-            Field field = findField(javaClass, fieldName, (colType == null ? fldType : colType));
+            Field field = findField(javaClass, fieldName, (collectionType == null ? fldType : collectionType));
             if (field == null) {
                 throw new MappingException(
                         "mapping.fieldNotAccessible", fieldName, javaClass.getName());
@@ -701,7 +747,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                 fldType = field.getType();
             }
 
-            typeInfoRef.typeInfo = getTypeInfo(fldType, colHandler, fldMap);
+            typeInfoRef.typeInfo = getTypeInfo(fldType, collectionHandler, fldMap);
 
             handler = new FieldHandlerImpl(field, typeInfoRef.typeInfo);
         } else if ((fldMap.getGetMethod() == null) && (fldMap.getSetMethod() == null)) {
@@ -711,8 +757,8 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                         "mapping.missingFieldName", javaClass.getName());
             }
 
-            List getSequence = new ArrayList();
-            List setSequence = new ArrayList();
+            List<Method> getSequence = new ArrayList<Method>();
+            List<Method> setSequence = new ArrayList<Method>();
             Method getMethod = null;
             Method setMethod = null;
 
@@ -735,12 +781,12 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                     }
                     getSequence.add(method);
 
-                    Class nextClass = method.getReturnType();
+                    Class<?> nextClass = method.getReturnType();
 
                     // Setter method for parent field
                     try {
                         methodName = SET_METHOD_PREFIX + capitalize(parentField);
-                        Class[] types = new Class[] {nextClass};
+                        Class<?>[] types = new Class[] {nextClass};
                         method = javaClass.getMethod(methodName, types);
                         if (isAbstractOrStatic(method)) { method = null; }
                     } catch (Exception ex) {
@@ -754,7 +800,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
 
                 // Find getter method for actual field
                 String methodName = GET_METHOD_PREFIX + capitalize( fieldName );
-                Class returnType = (colType == null) ? fldType : colType;
+                Class<?> returnType = (collectionType == null) ? fldType : collectionType;
                 getMethod = findAccessor(javaClass, methodName, returnType, true);
 
                 // If getMethod is null, check for boolean type method prefix
@@ -775,27 +821,27 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                 String isAccessor = IS_METHOD_PREFIX + capitalize(fieldName);
                 throw new MappingException("mapping.accessorNotFound",
                         getAccessor + "/" + isAccessor,
-                        (colType == null ? fldType : colType),
+                        (collectionType == null ? fldType : collectionType),
                         javaClass.getName());
             }
 
-            if ((fldType == null) && (colType == null)) {
+            if ((fldType == null) && (collectionType == null)) {
                 fldType = getMethod.getReturnType();
             }
 
             // We try to locate a set method anyway but complain only if we need one
             String methodName = SET_METHOD_PREFIX + capitalize(fieldName);
             setMethod = findAccessor(javaClass, methodName,
-                    (colType == null ? fldType : colType), false);
+                    (collectionType == null ? fldType : collectionType), false);
 
             // If we have a collection that need both set and get but we don't have a
             // set method, we fail
-            if ((setMethod == null) && (colType != null) && colRequireGetSet) {
+            if ((setMethod == null) && (collectionType != null) && colRequireGetSet) {
                 throw new MappingException("mapping.accessorNotFound", methodName,
-                        (colType == null ? fldType : colType), javaClass.getName());
+                        (collectionType == null ? fldType : collectionType), javaClass.getName());
             }
 
-            typeInfoRef.typeInfo = getTypeInfo(fldType, colHandler, fldMap);
+            typeInfoRef.typeInfo = getTypeInfo(fldType, collectionHandler, fldMap);
 
             fieldName = fldMap.getName();
             if (fieldName == null) {
@@ -811,9 +857,9 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
             Method[] setArray = null;
             if (getSequence.size() > 0) {
                 getArray = new Method[getSequence.size()];
-                getArray = (Method[]) getSequence.toArray(getArray);
+                getArray = getSequence.toArray(getArray);
                 setArray = new Method[setSequence.size()];
-                setArray = (Method[]) setSequence.toArray(setArray);
+                setArray = setSequence.toArray(setArray);
             }
 
             // Create handler
@@ -831,8 +877,8 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
 
             // First look up the get accessors
             if (fldMap.getGetMethod() != null) {
-                Class rtype = fldType;
-                if (colType != null) {
+                Class<?> rtype = fldType;
+                if (collectionType != null) {
                     String methodName = fldMap.getGetMethod();
                     if (methodName.startsWith(ENUM_METHOD_PREFIX)) {
                         // An enumeration method must really return a enumeration.
@@ -841,7 +887,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                         // An iterator method must really return a iterator.
                         rtype = Iterator.class;
                     } else {
-                        rtype = colType;
+                        rtype = collectionType;
                     }
                 }
 
@@ -851,7 +897,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                             fldMap.getGetMethod(), rtype, javaClass.getName());
                 }
 
-                if ((fldType == null) && (colType == null)) {
+                if ((fldType == null) && (collectionType == null)) {
                     fldType = getMethod.getReturnType();
                 }
             }
@@ -859,10 +905,10 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
             // Second look up the set/add accessor
             if (fldMap.getSetMethod() != null) {
                 String methodName = fldMap.getSetMethod();
-                Class type = fldType;
-                if (colType != null) {
+                Class<?> type = fldType;
+                if (collectionType != null) {
                     if (!methodName.startsWith(ADD_METHOD_PREFIX)) {
-                        type = colType;
+                        type = collectionType;
                     }
                 }
 
@@ -894,7 +940,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                 }
             }
 
-            typeInfoRef.typeInfo = getTypeInfo(fldType, colHandler, fldMap);
+            typeInfoRef.typeInfo = getTypeInfo(fldType, collectionHandler, fldMap);
 
             fieldName = fldMap.getName();
             if (fieldName == null) {
@@ -969,7 +1015,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
         return handler;
     }
 
-    private static boolean isAbstract(final Class cls) {
+    private static boolean isAbstract(final Class<?> cls) {
         return ((cls.getModifiers() & Modifier.ABSTRACT) != 0);
     }
 
@@ -978,7 +1024,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
             || ((method.getModifiers() & Modifier.STATIC) != 0);
     }
 
-    protected TypeInfo getTypeInfo(final Class fieldType,
+    protected TypeInfo getTypeInfo(final Class<?> fieldType,
                                    final CollectionHandler colHandler,
                                    final FieldMapping fieldMap) throws MappingException {
         return new TypeInfo(Types.typeFromPrimitive(fieldType), null, null,
@@ -996,8 +1042,8 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * @throws MappingException The field is not accessible or is not of the
      *         specified type.
      */
-    private final Field findField(final Class javaClass, final String fieldName,
-                                  Class fieldType) throws MappingException {
+    private final Field findField(final Class<?> javaClass, final String fieldName,
+                                  Class<?> fieldType) throws MappingException {
         try {
             // Look up the field based on its name, make sure it's only modifier
             // is public. If a type was specified, match the field type.
@@ -1011,8 +1057,8 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
             if (fieldType == null) {
                 fieldType = Types.typeFromPrimitive(field.getType());
             } else {
-                Class ft1 = Types.typeFromPrimitive(fieldType);
-                Class ft2 = Types.typeFromPrimitive(field.getType());
+                Class<?> ft1 = Types.typeFromPrimitive(fieldType);
+                Class<?> ft2 = Types.typeFromPrimitive(field.getType());
                 if ((ft1 != ft2) && (fieldType != Serializable.class)) {
                     throw new MappingException(
                             "mapping.fieldTypeMismatch", field, fieldType.getName());
@@ -1038,8 +1084,8 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * @throws MappingException The method is not accessible or is not of the
      *         specified type.
      */
-    public static final Method findAccessor(final Class javaClass,
-                                            final String methodName, Class fieldType,
+    public static final Method findAccessor(final Class<?> javaClass,
+                                            final String methodName, Class<?> fieldType,
                                             final boolean getMethod) throws MappingException {
         try {
             Method method = null;
@@ -1061,7 +1107,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                     fieldType = Types.typeFromPrimitive(method.getReturnType());
                 } else {
                     fieldType = Types.typeFromPrimitive(fieldType);
-                    Class returnType = Types.typeFromPrimitive(method.getReturnType());
+                    Class<?> returnType = Types.typeFromPrimitive(method.getReturnType());
 
                     //-- First check against whether the declared type is
                     //-- an interface or abstract class. We also check
@@ -1088,7 +1134,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                 // name. If the field type is know, look up a suitable method. If the
                 // field type is unknown, lookup the first method with that name and
                 // one parameter.
-                Class fieldTypePrimitive = null;
+                Class<?> fieldTypePrimitive = null;
                 if (fieldType != null) {
                     fieldTypePrimitive = Types.typeFromPrimitive(fieldType);
                     try {
@@ -1126,10 +1172,12 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                     Method[] methods = javaClass.getMethods();
                     for (int i = 0; i < methods.length; ++i) {
                         if (methods[i].getName().equals(methodName)) {
-                            Class[] paramTypes = methods[i].getParameterTypes();
-                            if (paramTypes.length != 1) { continue; }
+                            Class<?>[] paramTypes = methods[i].getParameterTypes();
+                            if (paramTypes.length != 1) { 
+                                continue; 
+                            }
 
-                            Class paramType = Types.typeFromPrimitive(paramTypes[0]);
+                            Class<?> paramType = Types.typeFromPrimitive(paramTypes[0]);
 
                             if (fieldType == null) {
                                 method = methods[i];
@@ -1146,7 +1194,9 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
                         }
                     }
 
-                    if (method == null) { return null; }
+                    if (method == null) { 
+                        return null; 
+                    }
                 }
             }
 
@@ -1166,7 +1216,9 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
 
     private static final String capitalize(final String name) {
         char first = name.charAt(0);
-        if (Character.isUpperCase(first)) { return name; }
+        if (Character.isUpperCase(first)) { 
+            return name; 
+        }
         return Character.toUpperCase(first) + name.substring(1);
     }
 
@@ -1174,30 +1226,32 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * Returns a list of column names that are part of the identity.
      *
      * @param ids Known identity names.
-     * @param clsMap Class mapping.
+     * @param clsMap The class mapping.
      * @return List of identity column names.
      */
     public static final String[] getIdentityColumnNames(final String[] ids,
-                                                        final ClassMapping clsMap) {
+            final ClassMapping clsMap) {
 
         String[] idNames = ids;
 
         if ((ids == null) || (ids.length == 0)) {
             ClassChoice classChoice = clsMap.getClassChoice();
-            if (classChoice == null) { classChoice = new ClassChoice(); }
+            if (classChoice == null) { 
+                classChoice = new ClassChoice(); 
+            }
 
             FieldMapping[] fieldMappings = classChoice.getFieldMapping();
 
-            List idNamesList = new ArrayList();
-            for (int i = 0; i < fieldMappings.length; i++) {
-                if (fieldMappings[i].getIdentity() == true) {
-                    idNamesList.add(fieldMappings[i].getName());
+            List<String> idNamesList = new ArrayList<String>();
+            for (FieldMapping fieldMapping : fieldMappings) {
+                if (fieldMapping.getIdentity() == true) {
+                    idNamesList.add(fieldMapping.getName());
                 }
             }
 
-            if (idNamesList.size() > 0) {
+            if (!idNamesList.isEmpty()) {
                 idNames = new String[idNamesList.size()];
-                idNames = (String[]) idNamesList.toArray(idNames);
+                idNames = idNamesList.toArray(idNames);
             }
         }
 
@@ -1210,7 +1264,7 @@ public abstract class AbstractMappingLoader extends AbstractMappingLoader2 {
      * @return true if the given class should be treated as a primitive
      * type
      */
-    protected static final boolean isPrimitive(final Class type) {
+    protected static final boolean isPrimitive(final Class<?> type) {
         if (type.isPrimitive()) { return true; }
         if ((type == Boolean.class) || (type == Character.class)) { return true; }
         return (type.getSuperclass() == Number.class);
