@@ -40,7 +40,7 @@
  *
  * Copyright 2000-2002 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: DateTimeBase.java 8145 2009-03-29 22:07:00Z rjoachim $
+ * $Id: DateTimeBase.java 8516 2010-01-25 12:14:04Z wguttmn $
  * Date         Author          Changes
  * 08/30/2001   Arnaud Blandin  added to Calendar() (patch from Sbastien Stormacq [S.Stormacq@aubay-si.lu])
  * 05/29/2001   Arnaud Blandin  Added order methods
@@ -67,7 +67,7 @@ import java.util.TimeZone;
  *
  * @author <a href="mailto:blandin@intalio.com">Arnaud Blandin</a>
  * @author <a href="mailto:edward.kuns@aspect.com">Edward Kuns</a>
- * @version $Revision: 8145 $
+ * @version $Revision: 8516 $
  * @see DateTime
  * @see Date
  * @see Time
@@ -1199,10 +1199,20 @@ public abstract class DateTimeBase implements java.io.Serializable, Cloneable {
             throw new ParseException(complaint + str + "\nThe Hour must be 2 digits long", idx);
         }
 
-        short value1;
+        short hour;
 
-        value1 = (short) ((chars[idx] - '0') * 10 + (chars[idx+1] - '0'));
-        result.setHour(value1);
+        hour = (short) ((chars[idx] - '0') * 10 + (chars[idx+1] - '0'));
+        
+        boolean isHourOfValue24 = false;
+        if (hour == 24) {
+        	result.setHour((short) 0);
+        	Duration oneDay = new Duration();
+            oneDay.setDay((short) 1);
+            result.addDuration(oneDay);
+            isHourOfValue24 = true;
+        } else {
+        	result.setHour(hour);
+        }
 
         idx += 2;
 
@@ -1217,8 +1227,12 @@ public abstract class DateTimeBase implements java.io.Serializable, Cloneable {
             throw new ParseException(complaint+str+"\nThe Minute must be 2 digits long", idx);
         }
 
-        value1 = (short) ((chars[idx] - '0') * 10 + (chars[idx+1] - '0'));
-        result.setMinute(value1);
+        short minutes = (short) ((chars[idx] - '0') * 10 + (chars[idx+1] - '0'));
+        
+        if (isHourOfValue24 && minutes != 0) {
+            throw new ParseException(complaint + str + "\nWhen an hour of 24 is used, minutes must be strictly of value 00.", idx);
+        }
+        result.setMinute(minutes);
 
         idx += 2;
 
@@ -1233,8 +1247,11 @@ public abstract class DateTimeBase implements java.io.Serializable, Cloneable {
             throw new ParseException(complaint + str + "\nThe Second must be 2 digits long", idx);
         }
 
-        value1 = (short) ((chars[idx] - '0') * 10 + (chars[idx+1] - '0'));
-        result.setSecond(value1);
+        short seconds = (short) ((chars[idx] - '0') * 10 + (chars[idx+1] - '0'));
+        if (isHourOfValue24 && seconds != 0) {
+            throw new ParseException(complaint + str + "\nWhen an hour of 24 is used, seconds must be strictly of value 00.", idx);
+        }
+        result.setSecond(seconds);
 
         idx += 2;
 
