@@ -45,7 +45,7 @@
  * All portions of this file developed by Keith Visco after Jan 19 2005 are
  * Copyright (C) 2005 Keith Visco. All Rights Reserved.
  *
- * $Id: XMLClassDescriptorImpl.java 8145 2009-03-29 22:07:00Z rjoachim $
+ * $Id: XMLClassDescriptorImpl.java 8734 2010-09-08 20:07:48Z wguttmn $
  */
 package org.exolab.castor.xml.util;
 
@@ -81,7 +81,7 @@ import org.exolab.castor.xml.util.resolvers.ResolveHelpers;
  * generated source code as well as the XMLMappingLoader.
  *
  * @author <a href="keith AT kvisco DOT com">Keith Visco</a>
- * @version $Revision: 8145 $ $Date: 2006-04-13 06:47:36 -0600 (Thu, 13 Apr 2006) $
+ * @version $Revision: 8734 $ $Date: 2006-04-13 06:47:36 -0600 (Thu, 13 Apr 2006) $
  */
 public class XMLClassDescriptorImpl extends Validator implements XMLClassDescriptor {
     /** The ALL compositor to signal the fields of the described class must all
@@ -109,7 +109,7 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
     private XMLFieldDescriptor[] _attArray = null;
 
     /** The Class that this ClassDescriptor describes. */
-    private Class _class = null;
+    private Class<?> _class = null;
 
     /** A variable to keep track of the number of container fields. */
     private int _containerCount = 0;
@@ -159,22 +159,22 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
     
     /** Defines the sequence of elements for unmarshalling validation
      *  (to be used with compositor == SEQUENCE only). */
-    private List _sequenceOfElements = new ArrayList();
+    private List<XMLFieldDescriptor> _sequenceOfElements = new ArrayList<XMLFieldDescriptor>();
     
-    private List _substitutes = new LinkedList();    
+    private List<String> _substitutes = new LinkedList<String>();    
 
     /** Map holding the properties set and read by Natures. */
-    private Map _properties = new HashMap();
+    private Map<String, Object> _properties = new HashMap<String, Object>();
     
     /** Map holding the available natures. */
-    private Set _natures = new HashSet();
+    private Set<String> _natures = new HashSet<String>();
 
     /**
      * Creates an XMLClassDescriptor class used by the Marshalling Framework.
      *
      * @param type the Class type with which this ClassDescriptor describes.
      */
-    public XMLClassDescriptorImpl(final Class type) {
+    public XMLClassDescriptorImpl(final Class<?> type) {
         this();
         if (type == null) {
             throw new IllegalArgumentException(NULL_CLASS_ERR);
@@ -189,7 +189,7 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
      *
      * @param type the Class type with which this ClassDescriptor describes.
      */
-    public XMLClassDescriptorImpl(Class type, String xmlName) {
+    public XMLClassDescriptorImpl(Class<?> type, String xmlName) {
         this();
 
         if (type == null) {
@@ -253,7 +253,7 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
      *         marshalled as XML attributes.
      */
     public XMLFieldDescriptor[]  getAttributeDescriptors() {
-        return (XMLFieldDescriptor[]) getAttributeArray().clone();
+        return getAttributeArray().clone();
     } // getAttributeDescriptors
 
     /**
@@ -275,7 +275,7 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
      * that should be marshalled as XML elements.
      */
     public XMLFieldDescriptor[]  getElementDescriptors() {
-        return (XMLFieldDescriptor[]) getElementArray().clone();
+        return getElementArray().clone();
     } // getElementDescriptors
     
     /**
@@ -288,11 +288,11 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
             final XMLFieldDescriptor elementDescriptor, UnmarshalState parentState, String xmlName) throws ValidationException {
         if (_compositor == SEQUENCE && _sequenceOfElements.size() > 0) {
         	
-        	if (parentState._expectedIndex == _sequenceOfElements.size() ) {
+        	if (parentState.getExpectedIndex() == _sequenceOfElements.size() ) {
         		throw new ValidationException ("Element with name " + xmlName + " passed to type " + getXMLName() + " in incorrect order; It is not allowed to be the last element of this sequence!");
         	}
         	
-        	XMLFieldDescriptor expectedElementDescriptor = (XMLFieldDescriptor) _sequenceOfElements.get(parentState._expectedIndex);
+        	XMLFieldDescriptor expectedElementDescriptor = _sequenceOfElements.get(parentState.getExpectedIndex());
             
             String expectedElementName = expectedElementDescriptor.getXMLName();
             String elementName = xmlName;
@@ -303,44 +303,44 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
             if (!anyNode && expectedElementDescriptor.getXMLName().equals("-error-if-this-is-used-")) {
             	
             	// find possible names
-            	ArrayList possibleNames = new ArrayList();
+            	List<String> possibleNames = new ArrayList<String>();
             	fillPossibleNames(possibleNames, expectedElementDescriptor);
             	
             	// check name
             	if (!possibleNames.contains(elementName)) {
             		if (!expectedElementDescriptor.isRequired()) {
-            			 parentState._expectedIndex++;
+            			 parentState.setExpectedIndex(parentState.getExpectedIndex() + 1);
                          checkDescriptorForCorrectOrderWithinSequence(elementDescriptor, parentState, xmlName);
             		} else {
             			throw new ValidationException ("Element with name " + elementName + " passed to type " + getXMLName() + " in incorrect order; expected element has to be member of the expected choice.");
             		}
             	} else {
-            		parentState._expectedIndex++;
+            		parentState.setExpectedIndex(parentState.getExpectedIndex() + 1);
             	}
                 return;
             }
 
             // multi valued flag
-            if (expectedElementDescriptor.isMultivalued() && !parentState._withinMultivaluedElement) {
-            	parentState._withinMultivaluedElement = true;
+            if (expectedElementDescriptor.isMultivalued() && !parentState.isWithinMultivaluedElement()) {
+            	parentState.setWithinMultivaluedElement(true);
             }
             
             if (!anyNode && !(expectedElementName).equals(elementName)) {
                 
                 // handle substitution groups !!! 
-                List substitutes =  expectedElementDescriptor.getSubstitutes();
+                List<String> substitutes =  expectedElementDescriptor.getSubstitutes();
                 if (substitutes != null && !substitutes.isEmpty()) {
                     if (substitutes.contains(elementName)) {
-                        if (!parentState._withinMultivaluedElement) {
-                            parentState._expectedIndex++;
+                        if (!parentState.isWithinMultivaluedElement()) {
+                            parentState.setExpectedIndex(parentState.getExpectedIndex() + 1);
                         }
                         return;
                     }
                 }
                 // handle multi-valued fields
                 if (expectedElementDescriptor.isMultivalued()) {
-                	parentState._withinMultivaluedElement = false;
-                    parentState._expectedIndex++;
+                	parentState.setWithinMultivaluedElement(false);
+                    parentState.setExpectedIndex(parentState.getExpectedIndex() + 1);
                     checkDescriptorForCorrectOrderWithinSequence(elementDescriptor, parentState, xmlName);
                     return;
                 }
@@ -350,32 +350,32 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
                 }
                 
                 // non required field, proceed until next required field
-                parentState._expectedIndex++;
+                parentState.setExpectedIndex(parentState.getExpectedIndex() + 1);
                 checkDescriptorForCorrectOrderWithinSequence(elementDescriptor, parentState, xmlName);
                 return;
                 
             }
             
-            if (!parentState._withinMultivaluedElement) {
-                parentState._expectedIndex++;
+            if (!parentState.isWithinMultivaluedElement()) {
+                parentState.setExpectedIndex(parentState.getExpectedIndex() + 1);
             }
         }
    }
     
     
-    private void fillPossibleNames(List possibleNames, XMLFieldDescriptor descriptor) {
+    private void fillPossibleNames(List<String> possibleNames, XMLFieldDescriptor descriptor) {
     	XMLFieldDescriptor[] descriptors = ((XMLClassDescriptor)descriptor.getClassDescriptor()).getElementDescriptors();
     	if (descriptors.length == 0) {
     		return;
     	}
-    	for (int i = 0; i < descriptors.length; i++) {
-    		if ("_items".equals(descriptors[i].getFieldName() ) 
-    				|| "-error-if-this-is-used-".equals(descriptors[i].getXMLName())) {
-        		fillPossibleNames(possibleNames, descriptors[i]);
-        	} else {
-        		possibleNames.add(descriptors[i].getXMLName());
-        	}
-    	}
+    	for (XMLFieldDescriptor fieldDescriptor : descriptors) {
+    	    if ("_items".equals(fieldDescriptor.getFieldName() ) 
+    	            || "-error-if-this-is-used-".equals(fieldDescriptor.getXMLName())) {
+    	        fillPossibleNames(possibleNames, fieldDescriptor);
+    	    } else {
+    	        possibleNames.add(fieldDescriptor.getXMLName());
+    	    }
+        }
     }
 
     /**
@@ -837,9 +837,9 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
         if (object == null) {
             throw new ValidationException("Cannot validate a null object.");
         }
-        Class a = getJavaClass();
+        Class<?> a = getJavaClass();
         ClassLoader acl = a.getClassLoader();
-        Class b = object.getClass();
+        Class<? extends Object> b = object.getClass();
         ClassLoader bcl = b.getClassLoader();
         if (!getJavaClass().isAssignableFrom(object.getClass())) {
             String err = "The given object is not an instance of the class"
@@ -1087,7 +1087,7 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
      *
      * @return The Java class
      */
-    public Class getJavaClass() {
+    public Class<?> getJavaClass() {
         return _class;
     } //-- getJavaClass
 
@@ -1251,7 +1251,7 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
                         }
                         //special case for collection
                         if (tempObj instanceof Collection) {
-                           result = ((Collection)tempObj).isEmpty();
+                           result = ((Collection) tempObj).isEmpty();
                         }
                     }
                 }
@@ -1275,7 +1275,7 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
      * @return true if the given class should be treated as a primitive
      * type
     **/
-    static boolean isPrimitive(Class type) {
+    static boolean isPrimitive(Class<?> type) {
 
         if (type == null) return false;
 
@@ -1302,7 +1302,7 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
     static boolean isDefaultPrimitiveValue(Object value) {
         if (value == null) return false;
 
-        Class type = value.getClass();
+        Class<?> type = value.getClass();
         if (type.isPrimitive()) {
             try {
                 return (value.equals(type.newInstance()));
@@ -1335,7 +1335,7 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
      *
      * @param type the Class type being described
      */
-    public void setJavaClass(Class type) {
+    public void setJavaClass(Class<?> type) {
         this._class = type;
     } //-- setJavaClass
 
@@ -1440,14 +1440,12 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
     protected void addSequenceElement(XMLFieldDescriptor element) {
         _sequenceOfElements.add(element);
     }
-
     
-    public List getSubstitutes()
-    {
+    public List<String> getSubstitutes() {
         return _substitutes;
     }
 
-    public void setSubstitutes(List substitutes) {
+    public void setSubstitutes(List<String> substitutes) {
         _substitutes = substitutes;
     }
          
@@ -1499,4 +1497,4 @@ public class XMLClassDescriptorImpl extends Validator implements XMLClassDescrip
         return _natures.contains(nature);
     }
     
-} //-- XMLClassDescriptor
+}
