@@ -45,7 +45,7 @@
  * All portions of this file developed by Keith Visco after Jan 19 2005 are
  * Copyright (C) 2005 Keith Visco. All Rights Reserved.
  *
- * $Id: XMLClassDescriptorResolverImpl.java 8186 2009-05-10 19:30:36Z wguttmn $
+ * $Id: XMLClassDescriptorResolverImpl.java 8691 2010-08-20 22:36:39Z wguttmn $
  */
 package org.exolab.castor.xml.util;
 
@@ -71,7 +71,7 @@ import org.exolab.castor.xml.util.resolvers.ResolveHelpers;
  * The default implementation of the ClassDescriptorResolver interface.
  *
  * @author <a href="mailto:keith AT kvisco DOT com">Keith Visco</a>
- * @version $Revision: 8186 $ $Date: 2006-04-25 15:08:23 -0600 (Tue, 25 Apr 2006) $
+ * @version $Revision: 8691 $ $Date: 2006-04-25 15:08:23 -0600 (Tue, 25 Apr 2006) $
  */
 public class XMLClassDescriptorResolverImpl implements XMLClassDescriptorResolver {
     /** 
@@ -169,10 +169,9 @@ public class XMLClassDescriptorResolverImpl implements XMLClassDescriptorResolve
     public void setMappingLoader(final MappingLoader mappingLoader) {
         _mappingLoader = mappingLoader;
         if (mappingLoader != null) {
-            Iterator<ClassDescriptor> descriptors = mappingLoader.descriptorIterator();
-            while (descriptors.hasNext()) {
-                XMLClassDescriptor descriptor = (XMLClassDescriptor) descriptors.next();
-                _descriptorCache.addDescriptor(descriptor.getJavaClass().getName(), descriptor);
+            for (ClassDescriptor classDescriptor : mappingLoader.getDescriptors()) {
+                _descriptorCache.addDescriptor(classDescriptor.getJavaClass().getName(), 
+                        (XMLClassDescriptor) classDescriptor);
             }
         }
     }
@@ -207,7 +206,7 @@ public class XMLClassDescriptorResolverImpl implements XMLClassDescriptorResolve
     /**
      * {@inheritDoc}
      */
-    public ClassDescriptor resolve(final Class type) throws ResolverException {
+    public ClassDescriptor resolve(final Class<?> type) throws ResolverException {
         if (type == null) {
             String message = "Type argument must not be null for resolve";
             LOG.warn(message);
@@ -403,6 +402,7 @@ public class XMLClassDescriptorResolverImpl implements XMLClassDescriptorResolve
        
     /**
      * {@inheritDoc}
+     * @deprecated
      */
     public void loadClassDescriptors(final String packageName) throws ResolverException {
         String message = "Already deprecated in the interface!";
@@ -648,7 +648,7 @@ public class XMLClassDescriptorResolverImpl implements XMLClassDescriptorResolve
                 String clsName = iter.next();
                 this.addDescriptor(clsName, (XMLClassDescriptor) descriptors.get(clsName));
             }
-        } //-- addAllDescriptors
+        }
     } // -- DescriptorCacheImpl
     
     /**
